/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2011
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4BookCachePage.m
 * Module:		UI
 * Library:		S4 iOS_UI Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import "S4BookCachePage.h"
#import "S4CommonDefines.h"


// ================================== Defines ==========================================



// ================================== Typedefs ==========================================



// ================================== Globals ==========================================



// ============================= Forward Declarations ==================================



// ================================== Inlines ==========================================




// ========================= Begin Class S4BookCachePage () ============================

@interface S4BookCachePage ()

@property (nonatomic, assign, readwrite) NSUInteger					pageIndex;

@end



// =================== Begin Class S4BookCachePage (PrivateImpl) =======================

@interface S4BookCachePage (PrivateImpl)

- (void)setImageRef: (CGImageRef)newImageRef;

@end



// ========================== Begin Class S4BookCachePage ==============================

@implementation S4BookCachePage


//============================================================================
//	S4BookCachePage synthesize properties
//============================================================================
@synthesize pageImage = m_pageImageRef;
@synthesize pageIndex = m_pageIndex;
@synthesize dataSource = m_dataSource;


//============================================================================
//	S4PdfPage :: init
//============================================================================
- (id)init
{
	self = [super init];
	if (nil != self)
	{
		m_pageImageRef = NULL;
		m_pageIndex = 0;
		m_dataSource = nil;
	}
	return (self);
}


//============================================================================
//	S4BookCachePage :: dealloc
//============================================================================
- (void)dealloc
{
	CGImageRef				oldImageRef;

	NS_SAFE_RELEASE(m_dataSource);
	if (NULL != m_pageImageRef)
	{
		oldImageRef = m_pageImageRef;
		m_pageImageRef = NULL;
		CGImageRelease(oldImageRef);
	}

	[super dealloc];
}


//============================================================================
//	S4BookCachePage :: setImage:
//============================================================================
- (BOOL)setImage: (CGImageRef)cgImage forPage: (NSUInteger)newPageIndex
{
	[self setImageRef: cgImage];
	self.pageIndex = newPageIndex;
	return (YES);
}


//============================================================================
//	S4BookCachePage :: isEqualToBookCachePage:
//============================================================================
- (BOOL)isEqualToBookCachePage: (S4BookCachePage *)page
{
	if (IS_NOT_NULL(page))
	{
		if (self.pageIndex == page.pageIndex)
		{
			return (YES);
		}
	}
	return (NO);
}


//============================================================================
//	S4BookCachePage (PrivateImpl) :: setImageRef:
//============================================================================
- (void)setImageRef: (CGImageRef)newImageRef
{
	CGImageRef			oldImageRef;
	
	// pointer comparison
	if (m_pageImageRef != newImageRef)
	{
		oldImageRef = m_pageImageRef;
		if (NULL != newImageRef)
		{
			m_pageImageRef = CGImageRetain(newImageRef);
		}
		else
		{
			m_pageImageRef = NULL;
		}
		
		if (NULL != oldImageRef)
		{
			CGImageRelease(oldImageRef);
		}
	}
}

@end
