/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2011
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4GenericBookView.m
 * Module:		UI
 * Library:		S4 iOS_UI Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import "S4GenericBookView.h"
#import "S4BookCachePage.h"
#import "S4CommonDefines.h"


// ================================== Defines ==========================================



// ================================== Typedefs ==========================================



// ================================== Globals ==========================================



// ============================= Forward Declarations ==================================



// ================================== Inlines ==========================================



// ======================== Begin Class S4GenericBookView () ===========================

@interface S4GenericBookView () 


@end



@interface S4GenericBookView (PrivateImpl) 


@end



@implementation S4GenericBookView (PrivateImpl)


@end



#pragma mark -
#pragma mark Implementation

@implementation S4GenericBookView

@synthesize dataSource = m_dataSource;


//============================================================================
//	S4GenericBookView :: init
//============================================================================
- (id)init
{
    return ([self initWithFrame: CGRectZero]);
}


//============================================================================
//	S4GenericBookView :: initWithFrame:
//		Designated initializer
//============================================================================
- (id)initWithFrame: (CGRect)frame
{
	self = [super initWithFrame: frame];
    if (nil != self)
	{
		m_dataSource = nil;
    }
    return (self);
}


//============================================================================
//	S4GenericBookView :: documentPageCount
//============================================================================
- (NSUInteger)documentPageCount
{
	return ([self.dataSource pageCount]);
}


//============================================================================
//	S4GenericBookView :: getImageForPageIndex:
//============================================================================
- (CGImageRef)getImageForPageIndex: (NSUInteger)pageIndex forPageType: (PageImageType)pageType
{
	S4BookCachePage						*curBookCachePage;
	CGColorSpaceRef						colorSpace;
	CGContextRef						context;
	CGImageRef							imgResult = NULL;

	imgResult = [m_bookCache imageForIndex: pageIndex withPageSize: m_pageSize];
	if (NULL == imgResult)
	{
		colorSpace = CGColorSpaceCreateDeviceRGB();
		context = CGBitmapContextCreate(NULL, 
										m_pageSize.width, 
										m_pageSize.height, 
										8,						// bits per component
										m_pageSize.width * 4, 	// bytes per row
										colorSpace, 
										(kCGImageAlphaPremultipliedLast | kCGBitmapByteOrder32Big));
		CGColorSpaceRelease(colorSpace);
		CGContextClipToRect(context, CGRectMake(0, 0, m_pageSize.width, m_pageSize.height));

		CGContextSetInterpolationQuality(context, kCGInterpolationHigh);
		CGContextSetRenderingIntent(context, kCGRenderingIntentDefault);

		[self.dataSource renderPageAtIndex: pageIndex inContext: context];

		imgResult = CGBitmapContextCreateImage(context);
		CGContextRelease(context);

		if (NULL != imgResult)
		{
			curBookCachePage = [[S4BookCachePage alloc] init];
			if (YES == [curBookCachePage setImage: imgResult forPage: pageIndex])
			{
				[m_bookCache setBookCachePage: curBookCachePage withPageSize: m_pageSize];
			}
			[curBookCachePage release];
		}
	}
	return (imgResult);
}


@end
