/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2011
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4GenericBookViewController.m
 * Module:		UI
 * Library:		S4 iOS_UI Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import "S4GenericBookViewController.h"
#import "S4CommonDefines.h"


// ================================== Defines ==========================================



// ================================== Typedefs ==========================================



// ================================== Globals ==========================================



// ============================= Forward Declarations ==================================



// ================================== Inlines ==========================================



// ============= Begin Class S4GenericBookViewController (PrivateImpl) =================

@interface S4GenericBookViewController (PrivateImpl)

- (void)initS4GenericBookViewController;

@end


@implementation S4GenericBookViewController (PrivateImpl)

- (void)initS4GenericBookViewController
{
	m_genericBookView = [[S4GenericBookView alloc] initWithFrame: CGRectZero];
	m_genericBookView.mode = UIInterfaceOrientationIsPortrait(self.interfaceOrientation) ? OnePageViewMode : TwoPageViewMode;
	m_genericBookView.myViewController = self;
}

@end


@implementation S4GenericBookViewController

@synthesize bookView = m_genericBookView;


- (id)init
{
	return [self initWithNibName: nil bundle: nil];
}


- (id)initWithNibName: (NSString *)nibName bundle: (NSBundle *)nibBundle
{
	self = [super initWithNibName: nibName bundle: nibBundle];
	if (nil != self)
	{
		[self initS4GenericBookViewController];
	}
	return self;
}


- (void)awakeFromNib
{
	[super awakeFromNib];
	[self initS4GenericBookViewController];
}


- (void)dealloc
{
	NS_SAFE_RELEASE(m_genericBookView);

    [super dealloc];
}



#pragma mark  UIViewController methods

- (void)loadView
{
	[super loadView];

	m_genericBookView.frame = self.view.bounds;
	m_genericBookView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
	[self.view addSubview: m_genericBookView];
}


- (void)viewDidLoad
{
	[super viewDidLoad];
}



#pragma mark -
#pragma mark Interface rotation

- (BOOL)shouldAutorotateToInterfaceOrientation: (UIInterfaceOrientation)toInterfaceOrientation 
{
    return (YES);
}


- (void)willRotateToInterfaceOrientation: (UIInterfaceOrientation)toInterfaceOrientation duration: (NSTimeInterval)duration
{
    if (UIInterfaceOrientationIsPortrait(toInterfaceOrientation))
	{
        m_genericBookView.mode = OnePageViewMode;
    }
	else
	{
        m_genericBookView.mode = TwoPageViewMode;
    }
}



#pragma mark -
#pragma mark View Interaction

- (void)goToPage: (NSUInteger)newCurrentPageIndex
{
	m_genericBookView.currentPageIndex = newCurrentPageIndex;
}


@end