/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2011
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4PdfBaseViewController.m
 * Module:		UI
 * Library:		S4 iOS_UI Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import "S4PdfBaseViewController.h"
#import "S4PdfDocument.h"
#import "S4CommonDefines.h"


// ================================== Defines ==========================================



// ================================== Typedefs ==========================================



// ================================== Globals ==========================================



// ============================= Forward Declarations ==================================



// ================================== Inlines ==========================================



// =============== Begin Class S4PdfBaseViewController (PrivateImpl) ===================

@interface S4PdfBaseViewController (PrivateImpl)

- (void)initS4PdfBaseViewController;

@end


@implementation S4PdfBaseViewController (PrivateImpl)

//============================================================================
//	S4PdfBaseViewController (PrivateImpl) :: initS4PdfBaseViewController
//============================================================================
- (void)initS4PdfBaseViewController
{
	m_pdfDocument = nil;
	m_bIsPrinting = NO;
}

@end



// ====================== Begin Class S4PdfBaseViewController ==========================

@implementation S4PdfBaseViewController


//============================================================================
//	S4PdfBaseViewController synthesize properties
//============================================================================
@synthesize pdfDocument = m_pdfDocument;


//============================================================================
//	S4PdfBaseViewController :: init
//============================================================================
- (id)init
{
	return [self initWithNibName: nil bundle: nil];
}


- (id)initWithNibName: (NSString *)nibName bundle: (NSBundle *)nibBundle
{
	self = [super initWithNibName: nibName bundle: nibBundle];
	if (nil != self)
	{
		[self initS4PdfBaseViewController];
	}
	return self;
}


- (void)awakeFromNib
{
	[super awakeFromNib];
	[self initS4PdfBaseViewController];
}


- (void)dealloc
{
	NS_SAFE_RELEASE(m_pdfDocument);

    [super dealloc];
}



/*********************************************  UIViewController Methods *********************************************/

- (void)didReceiveMemoryWarning
{
	if (IS_NOT_NULL(m_pdfDocument))
	{
		[m_pdfDocument reloadData];
	}

	[super didReceiveMemoryWarning];
}



/*********************************************  Interface rotation Methods *********************************************/

- (BOOL)shouldAutorotateToInterfaceOrientation: (UIInterfaceOrientation)toInterfaceOrientation 
{
    return (YES);
}


- (void)willRotateToInterfaceOrientation: (UIInterfaceOrientation)toInterfaceOrientation duration: (NSTimeInterval)duration
{
	if (YES == m_bIsPrinting)
	{
		[[UIPrintInteractionController sharedPrintController] dismissAnimated: NO];
		m_bIsPrinting = NO;
	}
}



/*********************************************  View Interaction Methods *********************************************/

- (void)displayPrintDialogFromButton: (UIBarButtonItem *)btn animated: (BOOL)animated showPageRange: (BOOL)pageRangeVisible
{
	UIPrintInteractionController				*pic;

	pic = [m_pdfDocument getPrintDialogWithPageRange: pageRangeVisible];
	if (IS_NOT_NULL(pic))
	{
		m_bIsPrinting = YES;
		[pic presentFromBarButtonItem: btn animated: animated completionHandler:
			^(UIPrintInteractionController *printInteractionController, BOOL completed, NSError *error)
			{
				if ((NO == completed) && (IS_NOT_NULL(error)))
				{
					// UIPrintErrorDomain;
					S4DebugLog(@"printDocument encountered error: %@", error);
				}
				m_bIsPrinting = NO;
			}
		];
	}
}


- (void)displayPrintDialogFromRect: (CGRect)rect
							inView: (UIView *)containingView
						  animated: (BOOL)animated
					 showPageRange: (BOOL)pageRangeVisible
{
	UIPrintInteractionController				*pic;

	pic = [m_pdfDocument getPrintDialogWithPageRange: pageRangeVisible];
	if (IS_NOT_NULL(pic))
	{
		m_bIsPrinting = YES;
		[pic presentFromRect: rect inView: containingView animated: animated completionHandler:
			^(UIPrintInteractionController *printInteractionController, BOOL completed, NSError *error)
			{
				if ((NO == completed) && (IS_NOT_NULL(error)))
				{
					// UIPrintErrorDomain;
					S4DebugLog(@"printDocument encountered error: %@", error);
				}
				m_bIsPrinting = NO;
			}
		];
	}
}


- (void)handleWebURL: (NSURL *)url
{
	[[UIApplication sharedApplication] openURL: url];
}

@end