/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2011
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4PdfBookViewController.m
 * Module:		UI
 * Library:		S4 iOS_UI Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import "S4PdfBookViewController.h"
#import "S4PdfDocument.h"
#import "S4CommonDefines.h"


// ================================== Defines ==========================================



// ================================== Typedefs ==========================================



// ================================== Globals ==========================================



// ============================= Forward Declarations ==================================



// ================================== Inlines ==========================================



// =============== Begin Class S4PdfBookViewController (PrivateImpl) ===================

@interface S4PdfBookViewController (PrivateImpl)

- (void)initS4PdfBookViewController;

@end


@implementation S4PdfBookViewController (PrivateImpl)

//============================================================================
//	S4PdfBookViewController (PrivateImpl) :: initS4PdfBookViewController
//============================================================================
- (void)initS4PdfBookViewController
{
	m_pdfBookView = [[S4PdfBookView alloc] initWithFrame: CGRectZero];
	m_pdfBookView.mode = UIInterfaceOrientationIsPortrait(self.interfaceOrientation) ? OnePageViewMode : TwoPageViewMode;
	m_pdfBookView.myViewController = self;
}

@end



// ====================== Begin Class S4PdfBookViewController ==========================

@implementation S4PdfBookViewController


//============================================================================
//	S4PdfBookViewController synthesize properties
//============================================================================
@synthesize bookView = m_pdfBookView;


//============================================================================
//	S4PdfBookViewController :: init
//============================================================================
- (id)init
{
	return [self initWithNibName: nil bundle: nil];
}


- (id)initWithNibName: (NSString *)nibName bundle: (NSBundle *)nibBundle
{
	self = [super initWithNibName: nibName bundle: nibBundle];
	if (nil != self)
	{
		[self initS4PdfBookViewController];
	}
	return self;
}


- (void)awakeFromNib
{
	[super awakeFromNib];
	[self initS4PdfBookViewController];
}


- (void)dealloc
{
	NS_SAFE_RELEASE(m_pdfBookView);

    [super dealloc];
}



/*********************************************  UIViewController Methods *********************************************/

- (void)loadView
{
	[super loadView];

	m_pdfBookView.frame = self.view.bounds;
	m_pdfBookView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
	[self.view addSubview: m_pdfBookView];
}


- (void)viewDidLoad
{
	[super viewDidLoad];

	m_pdfBookView.pdfDocument = m_pdfDocument;
}


- (void)didReceiveMemoryWarning
{
	[super didReceiveMemoryWarning];
}



/*********************************************  Interface rotation Methods *********************************************/

- (BOOL)shouldAutorotateToInterfaceOrientation: (UIInterfaceOrientation)toInterfaceOrientation 
{
    return (YES);
}


- (void)willRotateToInterfaceOrientation: (UIInterfaceOrientation)toInterfaceOrientation duration: (NSTimeInterval)duration
{
    if (UIInterfaceOrientationIsPortrait(toInterfaceOrientation))
	{
        m_pdfBookView.mode = OnePageViewMode;
    }
	else
	{
        m_pdfBookView.mode = TwoPageViewMode;
    }

	[super willRotateToInterfaceOrientation: toInterfaceOrientation duration: duration];
}



/*********************************************  View Interaction Methods *********************************************/

- (void)goToPage: (NSUInteger)newCurrentPageIndex
{
	m_pdfBookView.currentPageIndex = newCurrentPageIndex;
}


- (void)displayPrintDialogFromRect: (CGRect)rect animated: (BOOL)animated showPageRange: (BOOL)pageRangeVisible
{
	[self displayPrintDialogFromRect: rect inView: m_pdfBookView animated: animated showPageRange: pageRangeVisible];
}

@end