/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2011
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4PdfDocument.h
 * Module:		UI
 * Library:		S4 iOS_UI Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>


// =================================== Defines =========================================



// ================================== Typedefs =========================================



// ================================== Globals =========================================



// ============================= Forward Declarations =================================



// ================================== Protocols ========================================



// ============================== S4PdfDocument Class ==================================

@interface S4PdfDocument : NSObject <NSCoding>
{
}

// Properties
@property (nonatomic, assign, readonly) BOOL							isValidDocument;
@property (nonatomic, copy, readwrite) NSString							*password;
@property (nonatomic, strong, readonly) NSDate							*fileDate;
@property (nonatomic, strong, readonly) NSNumber						*fileSize;
@property (nonatomic, strong, readwrite) NSNumber						*pageNumber;
@property (nonatomic, assign, readonly) NSUInteger						pageCount;
@property (nonatomic, copy, readonly) NSString							*fileName;
@property (nonatomic, strong, readonly) NSURL							*pdfURL;

// Class methods
+ (BOOL)needPasswordForDocumentAtPath: (NSString *)pdfPathAndFilename;

// Instance methods
- (BOOL)openDocumentWithFilePath: (NSString *)fullFilePath usingPassword: (NSString *)passwordStr;
- (CGSize)getSizeOfPageAtIndex: (NSUInteger)pageIndex;
- (void)drawPageAtIndex: (NSUInteger)pageIndex inContext: (CGContextRef)context;
- (void)drawPageAtIndex: (NSUInteger)pageIndex inContext: (CGContextRef)context atPoint: (CGPoint)point;
- (void)drawPageAtIndex: (NSUInteger)pageIndex inContext: (CGContextRef)context atPoint: (CGPoint)point withScale: (float)scale;
- (void)drawPageAtIndex: (NSUInteger)pageIndex inContext: (CGContextRef)context inRect: (CGRect)displayRect;
- (void)drawPageFastAtIndex: (NSUInteger)pageIndex inContext: (CGContextRef)context inRect: (CGRect)displayRect;
- (CGPDFPageRef)getPageRefForPageIndex: (NSUInteger)pageIndex;
- (BOOL)reloadData;
- (UIPrintInteractionController *)getPrintDialogWithPageRange: (BOOL)pageRangeVisible;

@end
