/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2011
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4PageCurlViewController.m
 * Module:		UI
 * Library:		S4 iOS_UI Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import "S4PageCurlViewController.h"
#import "S4PageCurlModel.h"
#import "S4CommonDefines.h"


// ================================== Defines ==========================================



// ================================== Typedefs ==========================================



// ================================== Globals ==========================================



// ============================= Forward Declarations ==================================



// ================================== Inlines ==========================================



// ===================== Begin Class S4PageCurlViewController () =======================

@interface S4PageCurlViewController ()

@property (readwrite, strong, nonatomic) UIPageViewController			*pageViewController;

@end



// ============== Begin Class S4PageCurlViewController (PrivateImpl) ===================

@interface S4PageCurlViewController (PrivateImpl)

- (void)initS4PageCurlViewController;

@end



// ====================== Begin Class S4PageCurlViewController =========================

@implementation S4PageCurlViewController


//============================================================================
//	S4PageCurlViewController synthesize properties
//============================================================================
@synthesize modelController = m_modelController;
@synthesize pageViewController = m_pageViewController;


//============================================================================
//	S4PageCurlViewController :: init
//============================================================================
- (id)init
{
	return [self initWithNibName: nil bundle: nil];
}


//============================================================================
//	S4PageCurlViewController :: initWithNibName:
//============================================================================
- (id)initWithNibName: (NSString *)nibName bundle: (NSBundle *)nibBundle
{
	self = [super initWithNibName: nibName bundle: nibBundle];
	if (nil != self)
	{
		[self initS4PageCurlViewController];
	}
	return self;
}


//============================================================================
//	S4PageCurlViewController :: awakeFromNib
//============================================================================
- (void)awakeFromNib
{
	[super awakeFromNib];
	[self initS4PageCurlViewController];
}


//============================================================================
//	S4PageCurlViewController :: dealloc
//============================================================================
- (void)dealloc
{
	NS_SAFE_RELEASE(m_pageViewController);
	NS_SAFE_RELEASE(m_modelController);

	[super dealloc];
}



#pragma mark - View lifecycle

//============================================================================
//	S4PageCurlViewController :: viewDidLoad
//============================================================================
- (void)viewDidLoad
{
	UIViewController					*startingViewController;
	BOOL								bIsDoubleSided;
	UIViewController					*nextViewController;
	NSDictionary						*options;
	NSNumber							*spineNumberValue;
	NSArray								*viewControllers;
	CGRect								pageViewRect;

    [super viewDidLoad];

	// if the model controller is not set, then this VC cannot function
	if (IS_NOT_NULL(self.modelController))
	{
		startingViewController = [self.modelController viewControllerAtIndex: 0];
		if (UIInterfaceOrientationIsPortrait(self.interfaceOrientation))
		{
			bIsDoubleSided = NO;
			spineNumberValue = [NSNumber numberWithInteger: UIPageViewControllerSpineLocationMin];
			viewControllers = [NSArray arrayWithObject: startingViewController];
		}
		else
		{
			bIsDoubleSided = YES;
			spineNumberValue = [NSNumber numberWithInteger: UIPageViewControllerSpineLocationMid];
//			nextViewController = [self.modelController pageViewController: self.pageViewController
//										viewControllerAfterViewController: startingViewController];
			nextViewController = [self.modelController viewControllerAtIndex: 1];
			viewControllers = [NSArray arrayWithObjects: startingViewController, nextViewController, nil];
		}
		options = [NSDictionary dictionaryWithObject: spineNumberValue
											  forKey: UIPageViewControllerOptionSpineLocationKey];

		// now init the UIPageViewController
		self.pageViewController = [[[UIPageViewController alloc] initWithTransitionStyle: UIPageViewControllerTransitionStylePageCurl
																   navigationOrientation: UIPageViewControllerNavigationOrientationHorizontal
																				 options: options] autorelease];
		self.pageViewController.delegate = self;
		self.pageViewController.doubleSided = bIsDoubleSided;
		[self.pageViewController setViewControllers: viewControllers
										  direction: UIPageViewControllerNavigationDirectionForward
										   animated: NO
										 completion: NULL];
		self.pageViewController.dataSource = self.modelController;

		// set up the UIPageViewController in this parent ViewController 
		[self addChildViewController: self.pageViewController];
		[self.view addSubview: self.pageViewController.view];
		pageViewRect = self.view.bounds;
		pageViewRect = CGRectInset(pageViewRect, 15.0, 15.0);
		self.pageViewController.view.frame = pageViewRect;
		[self.pageViewController didMoveToParentViewController: self];
	}
}


//============================================================================
//	S4PageCurlViewController :: viewDidUnload
//============================================================================
- (void)viewDidUnload
{
    [super viewDidUnload];
}


//============================================================================
//	S4PageCurlViewController :: viewWillAppear:
//============================================================================
- (void)viewWillAppear: (BOOL)animated
{
    [super viewWillAppear:animated];
}


//============================================================================
//	S4PageCurlViewController :: viewDidAppear:
//============================================================================
- (void)viewDidAppear: (BOOL)animated
{
    [super viewDidAppear:animated];
}


//============================================================================
//	S4PageCurlViewController :: viewWillDisappear:
//============================================================================
- (void)viewWillDisappear: (BOOL)animated
{
	[super viewWillDisappear:animated];
}


//============================================================================
//	S4PageCurlViewController :: viewDidDisappear:
//============================================================================
- (void)viewDidDisappear: (BOOL)animated
{
	[super viewDidDisappear:animated];
}


//============================================================================
//	S4PageCurlViewController :: shouldAutorotateToInterfaceOrientation:
//============================================================================
- (BOOL)shouldAutorotateToInterfaceOrientation: (UIInterfaceOrientation)interfaceOrientation
{
	if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone)
	{
	    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
	}
	else
	{
	    return YES;
	}
}


//============================================================================
//	S4PageCurlViewController :: didReceiveMemoryWarning
//============================================================================
- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}



#pragma mark - UIPageViewController delegate methods

//============================================================================
//	S4PageCurlViewController :: didFinishAnimating:
//============================================================================
- (void)pageViewController: (UIPageViewController *)pageViewController
		didFinishAnimating: (BOOL)finished
   previousViewControllers: (NSArray *)previousViewControllers
	   transitionCompleted: (BOOL)completed
{
}


//============================================================================
//	S4PageCurlViewController :: spineLocationForInterfaceOrientation:
//============================================================================
- (UIPageViewControllerSpineLocation)pageViewController: (UIPageViewController *)pageViewController
				   spineLocationForInterfaceOrientation: (UIInterfaceOrientation)orientation
{
	UIViewController							*currentViewController;
	NSArray										*viewControllers = nil;
	NSUInteger									currentVCIndex;
	UIViewController							*nextViewController;
	UIViewController							*previousViewController;
	UIPageViewControllerSpineLocation			spineLocation;

	currentViewController = [self.pageViewController.viewControllers objectAtIndex: 0];
	if (UIInterfaceOrientationIsPortrait(orientation) ||
		([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone))
	{
	    // In portrait orientation or on iPhone: Set the spine position to "min" and the page view
		// controller's view controllers array to 
		// contain just one view controller. Setting the spine position to 
		// 'UIPageViewControllerSpineLocationMid' in landscape orientation
		// sets the doubleSided property to YES, so set it to NO here.
		viewControllers = [NSArray arrayWithObject: currentViewController];
		self.pageViewController.doubleSided = NO;
		spineLocation = UIPageViewControllerSpineLocationMin;
	}
	else
	{
		// In landscape orientation: Set set the spine location to "mid" and the page view controller's
		// view controllers array to contain two view controllers. If the current page is even,
		// set it to contain the current and next view controllers; if it is odd, set the array
		// to contain the previous and current view controllers.
		currentVCIndex = [self.modelController indexOfViewController: currentViewController];
		if ((0 == currentVCIndex) || (0 == (currentVCIndex % 2)))
		{
			nextViewController = [self.modelController pageViewController: self.pageViewController
										viewControllerAfterViewController: currentViewController];
			viewControllers = [NSArray arrayWithObjects: currentViewController, nextViewController, nil];
		}
		else
		{
			previousViewController = [self.modelController pageViewController: self.pageViewController
										   viewControllerBeforeViewController: currentViewController];
			viewControllers = [NSArray arrayWithObjects: previousViewController, currentViewController, nil];
		}
		self.pageViewController.doubleSided = YES;
		spineLocation = UIPageViewControllerSpineLocationMid;
	}
	[self.pageViewController setViewControllers: viewControllers
									  direction: UIPageViewControllerNavigationDirectionForward
									   animated: YES
									 completion: NULL];
	return (spineLocation);
}


//============================================================================
//	S4PageCurlViewController (PrivateImpl) :: currentPageIndex
//============================================================================
- (NSUInteger)currentPageIndex
{
	UIViewController						*currentViewController;

	currentViewController = [self.pageViewController.viewControllers objectAtIndex: 0];
	return ([self.modelController indexOfViewController: currentViewController]);
}


//============================================================================
//	S4PageCurlViewController (PrivateImpl) :: initS4PageCurlViewController
//============================================================================
- (void)initS4PageCurlViewController
{
	m_modelController = nil;
	m_pageViewController = nil;
}

@end
