//
//  ExamplesViewController.m
//  Leaves
//
//  Created by Tom Brow on 4/20/10.
//  Copyright 2010 Tom Brow. All rights reserved.
//

#import "ExamplesViewController.h"
#import "PDFExampleViewController.h"
#import "ImageExampleViewController.h"
#import "ProceduralExampleViewController.h"
#import "OverlayExampleViewController.h"
#import "S4PageCurlViewController.h"
#import "S4PageCurlDemoModel.h"
#import "S4PdfDocument.h"


enum {PDF, PDF_MW, IMAGE, PROCEDURAL, OVERLAY, PAGECURL_DEMO, PAGECURL_PDF, NUM_EXAMPLES};

@implementation ExamplesViewController

- (id)init {
    if ((self = [super initWithStyle:UITableViewStyleGrouped])) {
    }
    return self;
}

- (void)dealloc {
    [super dealloc];
}


- (BOOL) shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation
{
    return YES;
}


#pragma mark UITableViewDataSource methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}


- (NSInteger)tableView: (UITableView *)tableView numberOfRowsInSection: (NSInteger)section
{
    return NUM_EXAMPLES;
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }
    
	switch (indexPath.row) {
		case PDF: cell.textLabel.text = @"PDF example"; break;
		case PDF_MW: cell.textLabel.text = @"PDF MW example"; break;
		case IMAGE: cell.textLabel.text = @"Image example"; break;
		case PROCEDURAL: cell.textLabel.text = @"Procedural example"; break;
		case OVERLAY: cell.textLabel.text = @"Overlay example"; break;
		case PAGECURL_DEMO: cell.textLabel.text = @"Page Curl Demo example"; break;
		case PAGECURL_PDF: cell.textLabel.text = @"Page Curl PDF example"; break;
		default: cell.textLabel.text = @"";
	}    
    return cell;
}

#pragma mark UITableViewDelegate methods

- (void)tableView: (UITableView *)tableView didSelectRowAtIndexPath: (NSIndexPath *)indexPath
{
	UIViewController					*viewController;
	PDFExampleViewController			*pdfExampleViewController;
	NSString							*bundleFilePath;
	S4PdfDocument						*pdfDocument;
	S4PageCurlViewController			*pageCurlViewController;

	switch (indexPath.row)
	{
		case PDF: 
			pdfExampleViewController = [[[PDFExampleViewController alloc] init] autorelease];
			bundleFilePath = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent: @"paper.pdf"];
			if (NO == [S4PdfDocument needPasswordForDocumentAtPath: bundleFilePath])
			{
				pdfDocument = [[S4PdfDocument alloc] init];
				if (nil != pdfDocument)
				{
					if (YES == [pdfDocument openDocumentWithFilePath: bundleFilePath usingPassword: nil])
					{
						pdfExampleViewController.pdfDocument = pdfDocument;
					}
				}
			}
			viewController = pdfExampleViewController;
			break;
		case PDF_MW: 
			pdfExampleViewController = [[[PDFExampleViewController alloc] init] autorelease];
			bundleFilePath = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent: @"paper_mw.pdf"];
			if (NO == [S4PdfDocument needPasswordForDocumentAtPath: bundleFilePath])
			{
				pdfDocument = [[S4PdfDocument alloc] init];
				if (nil != pdfDocument)
				{
					if (YES == [pdfDocument openDocumentWithFilePath: bundleFilePath usingPassword: nil])
					{
						pdfExampleViewController.pdfDocument = pdfDocument;
					}
				}
			}
			viewController = pdfExampleViewController;
			break;
		case IMAGE: 
			viewController = [[[ImageExampleViewController alloc] init] autorelease]; 
			break;
		case PROCEDURAL:
			viewController = [[[ProceduralExampleViewController alloc] init] autorelease]; 
			break;
		case OVERLAY:
			viewController = [[[OverlayExampleViewController alloc] initWithNibName: @"OverlayExampleViewController" bundle: nil] autorelease]; 
			break;
		case PAGECURL_DEMO:
			pageCurlViewController = [[[S4PageCurlViewController alloc] init] autorelease];
			pageCurlViewController.modelController = [[S4PageCurlDemoModel alloc] init];
			viewController = pageCurlViewController;
			break;
		case PAGECURL_PDF:
			viewController = [[[OverlayExampleViewController alloc] initWithNibName: @"OverlayExampleViewController" bundle: nil] autorelease]; 
			break;
		default: 
			viewController = [[[UIViewController alloc] init] autorelease];
			break;
	} 
	[self.navigationController pushViewController: viewController animated: YES];
}


@end

