//
//  PDFExampleViewController.m
//  Leaves
//
//  Created by Tom Brow on 4/19/10.
//  Copyright 2010 Tom Brow. All rights reserved.
//


#import "PDFExampleViewController.h"
#import "S4BookViewUtilities.h"
#import "S4PdfDocument.h"



@implementation PDFExampleViewController


- (id)init
{
	self = [super init];
    if (nil != self)
	{
    }
    return self;
}


- (void)dealloc
{
    [super dealloc];
}


- (void)willRotateToInterfaceOrientation: (UIInterfaceOrientation)toInterfaceOrientation duration: (NSTimeInterval)duration
{
    [super willRotateToInterfaceOrientation: toInterfaceOrientation duration: duration];
	
    [self displayPageNumber: m_pdfBookView.currentPageIndex + 1];
}


- (void)displayPageNumber: (NSUInteger)pageNumber
{
	NSUInteger					numberOfPages;
	NSString					*pageNumberStr;

    numberOfPages = m_pdfDocument.pageCount;
	pageNumberStr = [NSString stringWithFormat: @"Page %u of %u", pageNumber, numberOfPages];

    if (m_pdfBookView.mode == TwoPageViewMode)
	{
        if (pageNumber > numberOfPages)
		{
            pageNumberStr = [NSString stringWithFormat: @"Page %u of %u", pageNumber - 1, numberOfPages];
        }
		else if (pageNumber > 1)
		{
            pageNumberStr = [NSString stringWithFormat: @"Pages %u-%u of %u", pageNumber - 1, pageNumber, numberOfPages];
        }
    }
	self.navigationItem.title = pageNumberStr;
}



#pragma mark S4BookViewDelegate methods

- (void)bookview: (S4BookView *)theView willTurnToPageAtIndex: (NSUInteger)pageIndex
{
	[self displayPageNumber: pageIndex + 1];
}


- (void)bookview: (S4BookView *)theView zoomingToScale: (CGFloat)scale
{
}


- (void)bookview: (S4BookView *)theView doubleTapCurrentView: (CGFloat)zoomLevel
{
}



#pragma mark Layer Support
/*
// Draw the CGPDFPageRef into the layer at the correct scale
- (void)drawLayer: (CALayer *)layer inContext: (CGContextRef)context
{
	CGPDFPageRef					pdfPageLeft;
	CGPDFPageRef					pdfPageRight;

	pdfPageRight = CGPDFDocumentGetPage(pdf, m_genericBookView.currentPageIndex + 1);
	CGPDFPageRetain(pdfPageRight);

	// ******************************  Common setup code ******************************
	// First fill the background with white.
	CGContextSetRGBFillColor(context, 1.0, 1.0, 1.0, 1.0);
	CGContextFillRect(context, CGContextGetClipBoundingBox(context));
	CGContextSaveGState(context);

	// Flip the context so that the PDF page is rendered right side up
	CGContextTranslateCTM(context, 0.0, layer.bounds.size.height);
	CGContextScaleCTM(context, 1.0, -1.0);

	if (OnePageViewMode == m_genericBookView.mode)
	{
		CGContextConcatCTM(context, CGPDFPageGetDrawingTransform(pdfPageRight, kCGPDFCropBox, layer.bounds, 0, true));
		CGContextDrawPDFPage(context, pdfPageRight);	
	}
	else
	{
		pdfPageLeft = CGPDFDocumentGetPage(pdf, m_genericBookView.currentPageIndex);
		CGPDFPageRetain(pdfPageLeft);
		
		// Drawing left page resized
		CGRect leftPage = layer.bounds;
		leftPage.size.width = layer.bounds.size.width / 2;
		CGContextConcatCTM(context, CGPDFPageGetDrawingTransform(pdfPageLeft, kCGPDFCropBox, leftPage, 0, true));
		CGContextDrawPDFPage(context, pdfPageLeft);	

		// Drawing right page resized
		CGRect rightPage = layer.bounds;
		rightPage.size.width = layer.bounds.size.width / 2;
		rightPage.origin.x = layer.bounds.size.width / 2;
		CGContextConcatCTM(context, CGPDFPageGetDrawingTransform(pdfPageRight, kCGPDFCropBox, rightPage, 0, true));
		CGContextDrawPDFPage(context, pdfPageRight);
		CGPDFPageRelease(pdfPageLeft);
	}
	CGContextRestoreGState(context);
	CGPDFPageRelease(pdfPageRight);
}
*/


#pragma mark Page Moving Commands

- (void)nextPage
{
	[self goToPage: m_pdfBookView.currentPageIndex + 1];
	[self displayPageNumber: m_pdfBookView.currentPageIndex + 1];
}


- (void)previousPage
{
	if (OnePageViewMode == m_pdfBookView.mode)
	{
		[self goToPage: m_pdfBookView.currentPageIndex - 1];
		[self displayPageNumber: m_pdfBookView.currentPageIndex + 1];
	}
	else
	{
		[self goToPage: m_pdfBookView.currentPageIndex - 2];
		[self displayPageNumber: m_pdfBookView.currentPageIndex + 2];
	}
}



#pragma mark UIViewController

- (void)viewDidLoad
{
	[super viewDidLoad];

	m_pdfBookView.delegate = self;
	[self displayPageNumber: 1];

	UIBarButtonItem *previousPageButton = [[UIBarButtonItem alloc] initWithTitle:@"<" style:UIBarButtonItemStylePlain target:self action:@selector(previousPage)];
	UIBarButtonItem *nextPageButton = [[UIBarButtonItem alloc] initWithTitle:@">" style:UIBarButtonItemStylePlain target:self action:@selector(nextPage)];
	self.navigationItem.leftBarButtonItem = previousPageButton;
	self.navigationItem.rightBarButtonItem = nextPageButton;

	[previousPageButton release];
	[nextPageButton release];
}

@end
