/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2012
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4GCDUtilities.m
 * Module:		Application
 * Library:		S4 iOS Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import "S4GCDUtilities.h"
#import "S4SingletonClass.h"
#import "S4CommonDefines.h"


// =================================== Defines =========================================



// ================================== Typedefs =========================================



// =================================== Globals =========================================

const char					*kDefaultQueueFormatStr = "com.seastones.gcdqueue_%d";


// ============================= Forward Declarations ==================================



// ================================== Inlines ==========================================



// ========================= Begin Class S4GCDUtilities () =============================

@interface S4GCDUtilities ()

- (void)oneTimeInit;

@end



// ========================== Begin Class S4GCDUtilities ===============================

@implementation S4GCDUtilities
{
}


///////////////////////////////////// START SINGLETON METHODS /////////////////////////////////////


SYNTHESIZE_SINGLETON_CLASS(S4GCDUtilities)



////////////////////////////////////// CLASS METHODS ///////////////////////////////////////

//============================================================================
//	S4GCDUtilities :: globalDispatchQueueFromNSOperationQueuePriority:
//============================================================================
+ (dispatch_queue_t)globalDispatchQueueFromNSOperationQueuePriority: (NSOperationQueuePriority)priority
{
	dispatch_queue_t			queueResult;

	if ((NSOperationQueuePriorityVeryHigh == priority) || (NSOperationQueuePriorityHigh == priority))
	{
		queueResult = GLOBAL_GCD_QUEUE_PRIORITY_HIGH;
	}
	else if (NSOperationQueuePriorityNormal == priority)
	{
		queueResult = GLOBAL_GCD_QUEUE_PRIORITY_DEFAULT;
	}
	else if (NSOperationQueuePriorityLow == priority)
	{
		queueResult =GLOBAL_GCD_QUEUE_PRIORITY_LOW ;
	}
	else if (NSOperationQueuePriorityVeryLow == priority)
	{
		queueResult = GLOBAL_GCD_QUEUE_PRIORITY_BACKGROUND;
	}
	else
	{
		queueResult =GLOBAL_GCD_QUEUE_PRIORITY_DEFAULT ;
	}
	return (queueResult);
}



///////////////////////////////////// INSTANCE METHODS /////////////////////////////////////

//============================================================================
//	S4AppUtils () :: oneTimeInit
//============================================================================
- (void)oneTimeInit
{
}


//============================================================================
//	S4GCDUtilities :: createPrivateGCDQueueWithIntLabel:
//============================================================================
- (dispatch_queue_t)createPrivateGCDQueueWithIntLabel: (NSUInteger)labelUInteger isSerial: (BOOL)bSerial
{
	char					label[256];

	sprintf(label, kDefaultQueueFormatStr, labelUInteger);
	if (YES == bSerial)
	{
		return (dispatch_queue_create(label, DISPATCH_QUEUE_SERIAL));
	}
	return (dispatch_queue_create(label, DISPATCH_QUEUE_CONCURRENT));
}

@end
