/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2012
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Gabriel Handford on 7/23/09
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		NSBundle+S4Yajl.h
 * Module:		Data
 * Library:		S4 iOS Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import <Foundation/Foundation.h>
#import "S4YajlParser.h"


// =================================== Defines =========================================



// ================================== Typedefs =========================================



// =================================== Globals =========================================



// ============================= Forward Declarations ==================================



// ================================== Protocols ========================================



// ============================= Class NSBundle(S4Yajl) ================================

/*!
 Utilities for loading JSON from resource bundles.
 
 @code
 id JSONValue = [[NSBundle mainBundle] yajlJSONFromResource:@"kegs.json"];
 @endcode
 */
@interface NSBundle (S4Yajl)

/*!
 Load JSON from bundle.  
 @param resource Resource name with extension, for example, file.json
 @throws S4JSONParserException On parse error
 */
- (id)yajlJSONFromResource: (NSString *)resource;

/*!
 Load JSON from bundle.
 @param resource Resource name with extension, for example, file.json
 @param options Parser options
  - S4JSONParserOptionsNone: No options
  - S4JSONParserOptionsAllowComments: Javascript style comments will be allowed in the input (both /&asterisk; &asterisk;/ and //)
  - S4JSONParserOptionsCheckUTF8: Invalid UTF8 strings will cause a parse error
  - S4JSONParserOptionsStrictPrecision: If YES will force strict precision and return integer overflow error
 
 @param error Out error
 @result JSON value (NSArray, NSDictionary) or nil if errored
 */
- (id)yajlJSONFromResource: (NSString *)resource options: (S4JSONParserOptions)options error: (NSError **)error;

@end
