/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2012
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Created by Gabriel Handford on 7/23/09
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		NSBundle+S4Yajl.m
 * Module:		Data
 * Library:		S4 iOS Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import "NSBundle+S4Yajl.h"
#import "NSObject+S4Yajl.h"


// =================================== Defines =========================================



// ================================== Typedefs =========================================



// =================================== Globals =========================================



// ============================= Forward Declarations ==================================



// ================================== Inlines ==========================================



// ======================= Begin Class NSBundle (S4YajlUtils) ==========================

@interface NSBundle (S4YajlUtils)

/*!
 Load data from resource.
 @param resource Name of resource
 @result NSData
 */
- (NSData *)yajl_gh_loadDataFromResource: (NSString *)resource;

/*!
 Load string data from resource.
 @param resource Name of resource
 @result NSString
 */
- (NSString *)yajl_gh_loadStringDataFromResource: (NSString *)resource;

/*!
 Get URL for resource.
 @param resource Name of resource
 @result URL to resource
 */
- (NSURL *)yajl_gh_URLForResource: (NSString *)resource;

@end




@implementation NSBundle (S4YajlUtils)

- (NSData *)yajl_gh_loadDataFromResource: (NSString *)resource
{
	NSParameterAssert(resource);
	NSString *resourcePath = [self pathForResource:[resource stringByDeletingPathExtension] ofType:[resource pathExtension]];
	if (!resourcePath) [NSException raise:NSInvalidArgumentException format:@"Resource not found: %@", resource];
	NSError *error = nil;
	NSData *data = [NSData dataWithContentsOfFile:resourcePath options:0 error:&error];
	if (error) [NSException raise:NSInvalidArgumentException format:@"Error loading resource at path (%@): %@", resourcePath, error];
	return data;
}


- (NSString *)yajl_gh_loadStringDataFromResource: (NSString *)resource
{
	return [[NSString alloc] initWithData:[self yajl_gh_loadDataFromResource:resource] encoding: NSUTF8StringEncoding];
}


- (NSURL *)yajl_gh_URLForResource: (NSString *)resource
{
	NSParameterAssert(resource);
	NSString *resourcePath = [self pathForResource:[resource stringByDeletingPathExtension] ofType:[resource pathExtension]];
	return resourcePath ? [NSURL fileURLWithPath:resourcePath] : nil;
}

@end




// ============================ Begin Class NSBundle (S4Yajl) =============================

@implementation NSBundle (S4Yajl)

- (id)yajlJSONFromResource: (NSString *)resource
{
  NSError *error = nil;
  id JSONValue = [self yajlJSONFromResource: resource options: S4JSONParserOptionsNone error: &error];
  if (error) [NSException raise: S4YajlParserException format:[error localizedDescription], nil];
  return JSONValue;
}


- (id)yajlJSONFromResource: (NSString *)resource options: (S4JSONParserOptions)options error: (NSError **)error
{
  return [[self yajl_gh_loadStringDataFromResource: resource] yajl_JSONWithOptions:S4JSONParserOptionsAllowComments error:error];
}

@end
