/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2012
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Created by Gabriel Handford on 7/23/09
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		NSObject+S4Yajl.m
 * Module:		Data
 * Library:		S4 iOS Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import "NSObject+S4Yajl.h"
#import "S4YajlSerializer.h"
#import "S4YajlJSONFile.h"


// =================================== Defines =========================================



// ================================== Typedefs =========================================



// =================================== Globals =========================================



// ============================= Forward Declarations ==================================



// ================================== Inlines ==========================================


// ========================== Begin Class NSObject (S4Yajl) ============================

@implementation NSObject (S4Yajl)


- (NSString *)yajl_JSONString
{
  return [self yajl_JSONStringWithOptions:S4JSONSerializerOptionsNone indentString:@"  "];
}


- (NSString *)yajl_JSONStringWithOptions: (S4JSONSerializerOptions)options indentString: (NSString *)indentString
{
  S4YajlSerializer *gen = [[S4YajlSerializer alloc] initWithGenOptions:options indentString:indentString];
  [gen object:self];
  NSString *buffer = [gen buffer];
  return buffer;
}


- (id)yajl_JSON
{
  NSError *error = nil;
  id JSON = [self yajl_JSON:&error];
  if (error)
	  [NSException raise: S4YajlParserException format:[error localizedDescription], nil];
  return JSON;
}


- (id)yajl_JSON: (NSError **)error
{
  return [self yajl_JSONWithOptions:S4JSONParserOptionsNone error:error];
}


- (id)yajl_JSONWithOptions: (S4JSONParserOptions)options error: (NSError **)error
{
	NSData *data = nil;

	if ([self isKindOfClass:[NSData class]])
	{
		data = (NSData *)self;
	}
	else if ([self respondsToSelector:@selector(dataUsingEncoding:)])
	{
		data = [(id)self dataUsingEncoding:NSUTF8StringEncoding];
	}
	else
	{
		[NSException raise: S4YajlParserException format:@"Object of type (%@) must implement dataUsingEncoding: to be parsed", [self class]];
	}

	S4YajlJSONFile *file = [[S4YajlJSONFile alloc] initWithParserOptions: options];
	[file parseData: data error: error];
	id root = file.rootJSONObject;
	return root;
}

@end
