/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2012
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4JSONCommon.h
 * Module:		Data
 * Library:		S4 iOS Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import <Foundation/Foundation.h>
#import "S4CommonDefines.h"


// =================================== Defines =========================================



// ================================== Typedefs =========================================

typedef NS_ENUM(NSInteger, S4JSONClassType)
{
	kInvalidJSONClass		= 0,
	kNSArrayJSONClass		= 1,
	kNSDictionaryJSONClass	= 2,
	kNSStringJSONClass		= 3,
	kNSNumberJSONClass		= 4,
	kUnknownJSONClass		= 5
};


typedef NS_OPTIONS(NSInteger, S4JSONParserOptions)
{
	S4JSONParserOptionsNone						= 0,
	S4JSONParserOptionsAllowComments			= 1 << 0,
	S4JSONParserOptionsCheckUTF8				= 1 << 1,
	S4JSONParserOptionsStrictPrecision			= 1 << 2,
	S4JSONParserOptionsUnicodeNewlines			= 1 << 3,
	S4JSONParserOptionsAllowTextAfterValidJSON	= 1 << 4
};


typedef NS_ENUM(NSInteger, S4JSONParserError)
{
	S4JSONParserNoError					= 0,
	S4JSONParserParsingError			= 1,
	S4JSONParserInvalidDataError		= 2,
	S4JSONParserAllocationError			= 3,
	S4JSONParserDoubleOverflowError		= 4,
	S4JSONParserIntegerOverflowError	= 5,
	S4JSONParserCanceledError			= 6,
	S4JSONParserUnknownError			= 7,
	S4JSONParserUnimplementedError		= 8
};


typedef NS_OPTIONS(NSInteger, S4JSONSerializerOptions)
{
	S4JSONSerializerOptionsNone						= 0,
	S4JSONSerializerOptionsBeautify					= 1 << 0,
	S4JSONSerializerOptionsIgnoreUnknownTypes		= 1 << 1,
	S4JSONSerializerOptionsIncludeUnsupportedTypes	= 1 << 2
};


// =================================== Globals =========================================

S4_EXTERN_CONSTANT_NSSTR				S4JSONCommonErrorDomain;


// ============================= Forward Declarations ==================================



// ================================== Protocols ========================================


// =============================== Class S4JSONCommon ==================================

@interface S4JSONCommon : NSObject
{
}

// Properties

// Class methods
+ (NSError *)errorforCode: (S4JSONParserError)code
			  description: (NSString *)descStr
				   reason: (NSString *)failStr;

+ (S4JSONClassType)classTypeForJSONObject: (id)jsonObject;

+ (NSDate *)dateFromJsonString: (NSString *)dateString;

// Instance methods

@end



