/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2012
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4JSONFile.h
 * Module:		Data
 * Library:		S4 iOS Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import <Foundation/Foundation.h>
#import "S4CommonDefines.h"
#import "S4JSONCommon.h"


// =================================== Defines =========================================

#define DEFAULT_JSON_TIMEOUT			(NSTimeInterval)300.0


// ================================== Typedefs =========================================



// =================================== Globals =========================================

S4_EXTERN_CONSTANT_NSSTR				S4JSONFileErrorDomain;
S4_EXTERN_CONSTANT_NSSTR				kDefaultJSONFileHostStr;


// ============================= Forward Declarations ==================================

@class S4JSONFile;


// ================================== Protocols ========================================

@protocol S4JSONFileDelegate <NSObject>

@required
// Called by the parser when parsing has begun.
- (void)jsonFileDidBeginParsingData: (S4JSONFile *)file;

// Called by the retriever in the case of an error.
- (void)jsonFile: (S4JSONFile *)file didFailWithError: (NSError *)error;

// Called by the retriever when all the JSON has been parsed.
- (void)jsonFile: (S4JSONFile *)file didEndParsingJSON: (id)jsonObject ofType: (S4JSONClassType)type;

@end


// ================================ Class S4JSONFile ===================================

@interface S4JSONFile : NSObject
{
@protected
	NSObject											*m_rootJSONObject;
	NSObject <S4JSONFileDelegate>						*m_delegate;
}

// Properties
@property (readonly, strong, nonatomic) NSObject				*rootJSONObject;
@property (readwrite, strong, nonatomic) NSObject				*userContext;

// Class methods
+ (id)jsonFileWithParserOptions: (S4JSONParserOptions)parserOptions;

// Instance methods
- (id)initWithParserOptions: (S4JSONParserOptions)parserOptions;

- (void)parseData: (NSData *)data error: (NSError **)error;

- (BOOL)requestJSONfromURLStr: (NSString *)urlStr
				  forDelegate: (NSObject <S4JSONFileDelegate> *)delegate
					  timeout: (NSTimeInterval)requestTimeout
				  useWebCache: (BOOL)bUseCache
				queuePriority: (NSOperationQueuePriority)priority
			useAsyncCallbacks: (BOOL)useAsync
			 useDispatchQueue: (dispatch_queue_t)dispatchQueue;

- (BOOL)parseJSONfromFilePath: (NSString *)pathStr
				  forDelegate: (NSObject <S4JSONFileDelegate> *)delegate
				queuePriority: (NSOperationQueuePriority)priority
			useAsyncCallbacks: (BOOL)useAsync
			 useDispatchQueue: (dispatch_queue_t)dispatchQueue;

@end
