/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2012
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4JSONFile.m
 * Module:		Data
 * Library:		S4 iOS Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import "S4JSONFile.h"
#import "S4FileUtilities.h"


// =================================== Defines =========================================



// ================================== Typedefs =========================================



// =================================== Globals =========================================

// ALL S4 LIBS SHOULD DEFINE THIS:
S4_INTERN_CONSTANT_NSSTR						S4JSONFileErrorDomain = @"S4JSONFileErrorDomain";


// ============================= Forward Declarations ==================================



// ================================== Inlines ==========================================



// =========================== Begin Class S4JSONFile () ===============================

@interface S4JSONFile ()

@end



// ============================== Begin Class S4JSONFile ===============================

@implementation S4JSONFile
{
	NSObject											*m_userContext;
}


//============================================================================
//	S4JSONFile :: properties
//============================================================================
@synthesize rootJSONObject = m_rootJSONObject;
@synthesize userContext = m_userContext;


//============================================================================
//	S4JSONFile :: jsonFileWithParserOptions:
//============================================================================
+ (id)jsonFileWithParserOptions: (S4JSONParserOptions)parserOptions
{
	return ([[[[self class] alloc] initWithParserOptions: parserOptions] autorelease]);
}


//============================================================================
//	S4JSONFile :: init
//============================================================================
- (id)init
{
	return ([self initWithParserOptions: S4JSONParserOptionsAllowComments]);
}


//============================================================================
//	S4JSONFile :: initWithParserOptions:
//============================================================================
- (id)initWithParserOptions: (S4JSONParserOptions)parserOptions
{
	self = [super init];
	if (nil != self)
	{
		// protected member vars
		m_rootJSONObject = nil;
		m_delegate = nil;
		m_userContext = nil;
	}
	return (self);
}


//============================================================================
//	S4JSONFile :: dealloc
//============================================================================
- (void)dealloc
{
	S4_NS_SAFE_RELEASE(m_rootJSONObject);
	S4_NS_SAFE_RELEASE(m_delegate);
	S4_NS_SAFE_RELEASE(m_userContext);

	[super dealloc];
}


//============================================================================
//	S4JSONFile :: parseData:
//============================================================================
- (void)parseData: (NSData *)data error: (NSError **)error
{
	if (nil != error)
	{
		*error = [S4JSONCommon errorforCode: S4JSONParserUnimplementedError
								description: nil
									 reason: nil];
	}
}


//============================================================================
//	S4JSONFile :: requestJSONfromURLStr:
//============================================================================
- (BOOL)requestJSONfromURLStr: (NSString *)urlStr
				  forDelegate: (NSObject <S4JSONFileDelegate> *)delegate
					  timeout: (NSTimeInterval)requestTimeout
				  useWebCache: (BOOL)bUseCache
				queuePriority: (NSOperationQueuePriority)priority
			useAsyncCallbacks: (BOOL)useAsync
			 useDispatchQueue: (dispatch_queue_t)dispatchQueue
{
	return (NO);
}


//============================================================================
//	S4JSONFile :: parseJSONfromFilePath:
//============================================================================
- (BOOL)parseJSONfromFilePath: (NSString *)pathStr
				  forDelegate: (NSObject <S4JSONFileDelegate> *)delegate
				queuePriority: (NSOperationQueuePriority)priority
			useAsyncCallbacks: (BOOL)useAsync
			 useDispatchQueue: (dispatch_queue_t)dispatchQueue
{
	return (NO);
}

@end
