/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2011
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4RegEx.m
 * Module:		Data
 * Library:		S4 iOS Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import "S4PropertiesDocument.h"
#import "CPropertyFactory.h"
#import "S4CommonDefines.h"


// =================================== Defines =========================================



// ================================== Typedefs =========================================



// =================================== Globals =========================================

static BOOL										g_bInitialized = NO;


// ============================= Forward Declarations ==================================



// ================================== Inlines ==========================================



// ======================= Begin Class S4PropertiesDocument () =========================

@interface S4PropertiesDocument ()

@property (readwrite, strong, nonatomic) NSMutableDictionary			*propertiesDictionary;

@end


// ======================== Begin Class S4PropertiesDocument ===========================

@implementation S4PropertiesDocument
{
	NSMutableDictionary					*m_propertiesDictionary;
}


//============================================================================
//	S4PropertiesDocument :: properties
//============================================================================
@synthesize propertiesDictionary = m_propertiesDictionary;



////////////////////////////////////////// CLASS METHODS //////////////////////////////////////////

//============================================================================
//	S4PropertiesDocument :: initialize
//============================================================================
+ (void)initialize
{
	if ((NO == g_bInitialized) && ([self class] == [S4PropertiesDocument class]))
	{
		@autoreleasepool
		{
			PropertyGetterBlock theGetter = ^(id _self, NSString *key)
			{ return([((S4PropertiesDocument *)_self).propertiesDictionary objectForKey: key]); };
			PropertySetterBlock theSetter = ^(id _self, NSString *key, id value)
			{ [((S4PropertiesDocument *)_self).propertiesDictionary setObject: value forKey: key]; };
			
			[[[CPropertyFactory alloc] init] configure: self getter: theGetter setter: theSetter error: NULL];
		}
		g_bInitialized = YES;
	}
}



//////////////////////////////////////// INSTANCE METHODS /////////////////////////////////////////

//============================================================================
//	S4PropertiesDocument :: init
//============================================================================
- (id)init
{
	return ([self initWithDictionary: nil]);
}


//============================================================================
//	S4PropertiesDocument :: initWithDictionary:
//============================================================================
- (id)initWithDictionary: (NSDictionary *)propsDictionary
{
	self = [super init];
	if (nil != self)
	{
		DECLARE_ABSTRACT_CLASS(S4PropertiesDocument)
		if (IS_NOT_NULL(propsDictionary))
		{
			m_propertiesDictionary = [[NSMutableDictionary alloc] initWithDictionary: propsDictionary
																		   copyItems: YES];
		}
		else
		{
			m_propertiesDictionary = [[NSMutableDictionary alloc] initWithCapacity: 1];
		}
	}
	return (self);
}


//============================================================================
//	S4PropertiesDocument :: dealloc
//============================================================================
- (void)dealloc
{
	S4_NS_SAFE_RELEASE(m_propertiesDictionary);
	
	[super dealloc];
}


//============================================================================
//	S4PropertiesDocument :: updatePropertiesFromDictionary:
//============================================================================
- (void)updatePropertiesFromDictionary: (NSDictionary *)propsDictionary fromFetch: (BOOL)wasFetched
{
	if (IS_NOT_NULL(propsDictionary))
	{
		[m_propertiesDictionary addEntriesFromDictionary: propsDictionary];
	}
}



/*********************************************  NSCoding Protocol Methods *********************************************/

//============================================================================
//	S4PropertiesDocument :: encodeWithCoder:
//============================================================================
- (void)encodeWithCoder: (NSCoder *)coder
{
	[coder encodeObject: m_propertiesDictionary forKey: @"propertiesDictionary"];
}


//============================================================================
//	S4PropertiesDocument :: initWithCoder:
//============================================================================
- (id)initWithCoder: (NSCoder *)coder
{
	NSDictionary				*propsDictionary;

	self = [super init];
	if (nil != self)
	{
		propsDictionary = (NSDictionary *)[coder decodeObjectForKey: @"propertiesDictionary"];
		if (IS_NOT_NULL(propsDictionary))
		{
			m_propertiesDictionary = [[NSMutableDictionary alloc] initWithDictionary: propsDictionary
																		   copyItems: YES];
		}
		else
		{
			m_propertiesDictionary = [[NSMutableDictionary alloc] initWithCapacity: 1];
		}
	}
	return (self);
}

@end
