/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2012
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4YajlParser.h
 * Module:		Data
 * Library:		S4 iOS Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import <Foundation/Foundation.h>
#import "S4CommonDefines.h"
#import "S4JSONCommon.h"


// =================================== Defines =========================================



// ================================== Typedefs =========================================



// =================================== Globals =========================================

S4_EXTERN_CONSTANT_NSSTR				S4YajlParserErrorDomain;
S4_EXTERN_CONSTANT_NSSTR				S4YajlParserException;
S4_EXTERN_CONSTANT_NSSTR				S4YajlParserValueKey;


// ============================= Forward Declarations ==================================

@class S4YajlParser;


// ================================== Protocols ========================================

@protocol S4YajlParserDelegate <NSObject>

@optional
- (void)parserDidStartDictionary: (S4YajlParser *)parser;
- (void)parserDidEndDictionary: (S4YajlParser *)parser;
- (void)parserDidStartArray: (S4YajlParser *)parser;
- (void)parserDidEndArray: (S4YajlParser *)parser;
- (void)parser: (S4YajlParser *)parser didMapKey: (NSString *)key;
- (void)parser: (S4YajlParser *)parser didAdd: (id)value;

@end


// ============================== Class S4YajlParser ====================================

@interface S4YajlParser : NSObject
{
}

@property (nonatomic, assign, readwrite) id <S4YajlParserDelegate>				delegate;
@property (nonatomic, retain, readonly) NSError									*parserError;
@property (nonatomic, assign, readonly) S4JSONParserOptions						parserOptions;
@property (nonatomic, assign, readonly) NSUInteger								bytesConsumed;

- (id)initWithParserOptions: (S4JSONParserOptions)parserOptions;
- (S4JSONParserError)parse: (NSData *)data;
- (S4JSONParserError)parseCompleted;

@end
