/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2012
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4NetUtilities.m
 * Module:		Network
 * Library:		S4 iOS Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import <CoreFoundation/CoreFoundation.h>
#import "S4NetUtilities.h"
#import "Base64Transcoder.h"
#import "S4CommonDefines.h"


// ================================== Defines ==========================================

#define DEFAULT_TIMEOUT_INTERVAL		(NSTimeInterval)120.0


// ================================== Typedefs ==========================================



// ================================== Globals ==========================================



// ============================= Forward Declarations ==================================



// ================================== Inlines ==========================================



// =========================== Begin Class S4NetUtilities (PrivateImpl) ======================

@interface S4NetUtilities (PrivateImpl)

+ (NSString *)urlEncodePostString: (NSString *)strToEncode;
+ (NSData *)encodedFormFromDictionary: (NSDictionary *)dict;

@end




@implementation S4NetUtilities (PrivateImpl)

//============================================================================
//	S4NetUtilities :: urlEncodePostString:
//============================================================================
+ (NSString *)urlEncodePostString: (NSString *)strToEncode
{
	CFStringRef				unencodedStr;
	NSString				*tmpEncodedStr;
	NSString				*encodedStrResult = nil;
	
	if (STR_NOT_EMPTY(strToEncode))
	{
		unencodedStr = CFURLCreateStringByReplacingPercentEscapesUsingEncoding(kCFAllocatorDefault,
																			   (CFStringRef)strToEncode,
																			   CFSTR(""),
																			   kCFStringEncodingUTF8);
		if (NULL != unencodedStr)
		{
			tmpEncodedStr = (NSString *)CFURLCreateStringByAddingPercentEscapes(kCFAllocatorDefault,
																				unencodedStr,
																				NULL,
																				CFSTR("!*'();:@&=+$,/?%#[] <>{}|^~`\"\\"),
																				kCFStringEncodingUTF8);
			if (STR_NOT_EMPTY(tmpEncodedStr))
			{
				encodedStrResult = [tmpEncodedStr autorelease];
			}
			CFRelease(unencodedStr);
		}
	}
	return (encodedStrResult);	
}


//============================================================================
//	S4NetUtilities (PrivateImpl) :: encodedFormFromDictionary:
//============================================================================
+ (NSData *)encodedFormFromDictionary: (NSDictionary *)dict
{
	NSEnumerator					*keys;
	NSString						*curKey;
	NSString						*curValue;
	NSString						*encodedKey;
	NSString						*encodedValue;
	BOOL							bNext;
	NSMutableString					*formStr;
	NSData							*formDataResult = nil;

	if (IS_NOT_NULL(dict))
	{
		keys = [dict keyEnumerator];
		formStr = [NSMutableString stringWithCapacity: 256];

		// loop to get the key-value pairs, properly escaped, in one string
		bNext = NO;
		curKey = [keys nextObject];
		while (IS_NOT_NULL(curKey))
		{
//			encodedKey = [S4NetUtilities urlEncodeParameterString: curKey];
			encodedKey = [S4NetUtilities urlEncodePostString: curKey];

			// now get the value
			curValue = [dict objectForKey: curKey];
			if (IS_NULL(curValue))
			{
				encodedValue = @"";
			}
			else
			{
//				encodedValue = [S4NetUtilities urlEncodeParameterString: [curValue description]];
				encodedValue = [S4NetUtilities urlEncodePostString: [curValue description]];
			}

			if (YES == bNext)
			{
				[formStr appendString: @"&"];
			}
			else
			{
				bNext = YES;
			}
			[formStr appendString: [NSString stringWithFormat: @"%@=%@", encodedKey, encodedValue]];
			curKey = [keys nextObject];
		}

		if (STR_NOT_EMPTY(formStr))
		{
			formDataResult = [formStr dataUsingEncoding: NSASCIIStringEncoding];
		}
	}
	return (formDataResult);
}

@end




// ============================== Begin Class S4NetUtilities =========================

@implementation S4NetUtilities

//============================================================================
//	S4NetUtilities :: urlEncodeParameterString:
//============================================================================
+ (NSString *)urlEncodeParameterString: (NSString *)strToEncode
{
	CFStringRef				unencodedStr;
	NSString				*tmpEncodedStr;
	NSString				*encodedStrResult = nil;
	
	if (STR_NOT_EMPTY(strToEncode))
	{
		unencodedStr = CFURLCreateStringByReplacingPercentEscapesUsingEncoding(kCFAllocatorDefault,
																			   (CFStringRef)strToEncode,
																			   CFSTR(""),
																			   kCFStringEncodingUTF8);
		if (NULL != unencodedStr)
		{
			tmpEncodedStr = (NSString *)CFURLCreateStringByAddingPercentEscapes(kCFAllocatorDefault,
																				unencodedStr,
																				CFSTR(":&=+/?"),
																				CFSTR("% '\"+<>^`@*[]{}#()!$;,\\"),
																				kCFStringEncodingUTF8);
			if (STR_NOT_EMPTY(tmpEncodedStr))
			{
				encodedStrResult = [tmpEncodedStr autorelease];
			}
			CFRelease(unencodedStr);
		}
	}
	return (encodedStrResult);	
}


//============================================================================
//	S4NetUtilities :: urlDecodeString:
//============================================================================
+ (NSString *)urlDecodeString: (NSString *)strToDecode
{
	CFStringRef				decodedStr;
	
	if (STR_NOT_EMPTY(strToDecode))
	{
		decodedStr = CFURLCreateStringByReplacingPercentEscapesUsingEncoding(kCFAllocatorDefault,
																			 (CFStringRef)strToDecode,
																			 CFSTR(""),
																			 kCFStringEncodingUTF8);
		if (NULL != decodedStr)
		{
			return ([(NSString *)decodedStr autorelease]);
		}
	}
	return (nil);
}


//============================================================================
//	S4NetUtilities : createNSUrlForPathStr
//============================================================================
+ (NSURL *)createNSUrlForPathStr: (NSString *)pathStr baseStr: (NSString *)baseStr
{
	NSString					*baseUrlStr;
	NSString					*pathUrlStr;
	CFURLRef					baseURL = NULL;
	BOOL						bError = NO;
	NSURL						*tmpUrl;
	NSURL						*urlResult = nil;

	// the base url string can be nil
	if (STR_NOT_EMPTY(baseStr))
	{
		baseUrlStr = [S4NetUtilities urlEncodeParameterString: baseStr];
		if (STR_NOT_EMPTY(baseUrlStr))
		{
			baseURL = CFURLCreateWithString(kCFAllocatorDefault, (CFStringRef)baseUrlStr, NULL);
		}

		// but if it was not nil, then baseURL being NULL means we had an error
		if (NULL == baseURL)
		{
			bError = YES;
		}
	}

	// we *must* have a valid path string, and either no base string or no error in creating a URL from the base string if passed in
	if ((STR_NOT_EMPTY(pathStr)) && (NO == bError))
	{
		pathUrlStr = [S4NetUtilities urlEncodeParameterString: pathStr];
		if (STR_NOT_EMPTY(pathUrlStr))
		{
			tmpUrl = (NSURL *)CFURLCreateWithString(kCFAllocatorDefault, (CFStringRef)pathUrlStr, baseURL);
			if (IS_NOT_NULL(tmpUrl))
			{
				urlResult = [tmpUrl autorelease];
			}
		}
	}

	if (NULL != baseURL)
	{
		CFRelease(baseURL);
	}
	return (urlResult);
}


//============================================================================
//	S4NetUtilities :: createRequestForPath:
//============================================================================
+ (NSURLRequest *)createRequestForPath: (NSString *)pathStr
							  useCache: (BOOL)bUseCache
					   timeoutInterval: (NSTimeInterval)timeoutInSec
							  postData: (NSData *)data
							dataIsForm: (BOOL)bIsForm
						 handleCookies: (BOOL)bHandleCookies
{
	NSURL						*nsURL;
	NSURLRequest				*requestResult = nil;
	
	if (STR_NOT_EMPTY(pathStr))
	{
		// create a properly escaped NSURL from the string params
		nsURL = [S4NetUtilities createNSUrlForPathStr: pathStr baseStr: nil];
		if (IS_NOT_NULL(nsURL))
		{
			// create the request
			requestResult = [S4NetUtilities createRequestForURL: nsURL
													   useCache: bUseCache
												timeoutInterval: timeoutInSec
													   postData: data
													 dataIsForm: bIsForm
												  handleCookies: bHandleCookies];
		}
	}
	return (requestResult);
}


//============================================================================
//	S4NetUtilities :: createSimpleRequestForURL:
//============================================================================
+ (NSMutableURLRequest *)createSimpleRequestForURL: (NSURL *)url useCache: (BOOL)bUseCache
{
	return ([S4NetUtilities createRequestForURL: url useCache: bUseCache timeoutInterval: 0.0 postData: nil dataIsForm: NO handleCookies: YES]);
}


//============================================================================
//	S4NetUtilities :: createPostRequestForURL:
//============================================================================
+ (NSMutableURLRequest *)createPostRequestForURL: (NSURL *)url
								 timeoutInterval: (NSTimeInterval)timeoutInSec
								  fromDictionary: (NSDictionary *)dict
								   handleCookies: (BOOL)bHandleCookies
{
	NSData					*dataFromDict;

	dataFromDict = [S4NetUtilities encodedFormFromDictionary: dict];
	return ([S4NetUtilities createRequestForURL: url
									   useCache: NO
								timeoutInterval: timeoutInSec
									   postData: dataFromDict
									 dataIsForm: YES
								  handleCookies: bHandleCookies]);
}


//============================================================================
//	S4NetUtilities :: createRequestForURL:
//============================================================================
+ (NSMutableURLRequest *)createRequestForURL: (NSURL *)url
									useCache: (BOOL)bUseCache
							 timeoutInterval: (NSTimeInterval)timeoutInSec
									postData: (NSData *)data
								  dataIsForm: (BOOL)bIsForm
							   handleCookies: (BOOL)bHandleCookies
{
	NSString						*postLength;
	NSMutableURLRequest				*requestResult = nil;

	if (IS_NOT_NULL(url))
	{
		requestResult = [[NSMutableURLRequest alloc] init];
		if (IS_NOT_NULL(requestResult))
		{
			[requestResult setURL: url];

			if (IS_NOT_NULL(data))
			{
				[requestResult setHTTPMethod: @"POST"];
				postLength = [NSString stringWithFormat: @"%d", [data length]];
				[requestResult setValue: postLength forHTTPHeaderField: @"Content-Length"];
				if (YES == bIsForm)
				{
					[requestResult setValue: @"application/x-www-form-urlencoded" forHTTPHeaderField: @"Content-Type"];
				}
				[requestResult setHTTPBody: data];
			}
			else
			{
				[requestResult setHTTPMethod: @"GET"];
			}

			if (YES == bUseCache)
			{
				[requestResult setCachePolicy: NSURLRequestUseProtocolCachePolicy];
			}
			else
			{
				[requestResult setCachePolicy: NSURLRequestReloadIgnoringLocalCacheData];
			}

			if (0 < timeoutInSec)
			{
				[requestResult setTimeoutInterval: timeoutInSec];
			}
			else
			{
				[requestResult setTimeoutInterval: DEFAULT_TIMEOUT_INTERVAL];
			}

			[requestResult setHTTPShouldHandleCookies: bHandleCookies];
			[requestResult setValue: @"gzip" forHTTPHeaderField: @"Accept-Encoding"];
			[requestResult setValue: @"Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)" forHTTPHeaderField: @"User-Agent"];
		}
	}
	return (requestResult);
}


//============================================================================
//	S4NetUtilities :: addBasicAuthToURLRequest:
//============================================================================
+ (NSMutableURLRequest *)addBasicAuthToURLRequest: (NSURLRequest *)urlRequest
									  forUserName: (NSString *)username
									  andPassword: (NSString *)password
{
	NSString						*dataStr;
	NSData							*encodeData;
	NSUInteger						rawLength;
	size_t							outBufSize;
	char							*encodeArray;
	NSString						*encodedStr;
	NSString						*authStr;
	NSMutableURLRequest				*requestResult = nil;
	
	if ((IS_NOT_NULL(urlRequest)) && (STR_NOT_EMPTY(username)) && (STR_NOT_EMPTY(password)))
	{
		dataStr = [NSString stringWithFormat: @"%@:%@", username, password];
		if (IS_NOT_NULL(dataStr))
		{
			encodeData = [dataStr dataUsingEncoding: NSUTF8StringEncoding];
			if (IS_NOT_NULL(encodeData))
			{
				// find the length, in bytes, of the username/password converted to bytes
				rawLength = [encodeData length];

				// now ask the Base64Encoder to estimate the output buffer size (adding a fudge factor)
				outBufSize = (EstimateBas64EncodedDataSize((size_t)rawLength)) + 16;

				// and calloc a buffer to hold the encoded bytes; calloc so result is NULL-terminated C string
				encodeArray = (char *)calloc(outBufSize, sizeof(char));
				if (NULL != encodeArray)
				{
					// Base64 encode username and password
					if (YES == Base64EncodeData([encodeData bytes], (size_t)rawLength, encodeArray, &outBufSize))
					{
						encodedStr = [NSString stringWithCString: (const char *)encodeArray encoding: NSUTF8StringEncoding];
						if (IS_NOT_NULL(encodedStr))
						{
							authStr = [NSString stringWithFormat: @"Basic %@", encodedStr];
							if (IS_NOT_NULL(authStr))
							{
								requestResult = [urlRequest mutableCopyWithZone: [urlRequest zone]];
								if (IS_NOT_NULL(requestResult))
								{
									[requestResult addValue: authStr forHTTPHeaderField: @"Authorization"];
								}
							}
						}
					}
					free(encodeArray);
				}
			}
		}
	}
	return (requestResult);
}


//============================================================================
//	S4NetUtilities :: createNSURLCredentialForUser:
//============================================================================
+ (NSURLCredential *)createNSURLCredentialForUser: (NSString *)username withPassword: (NSString *)password
{
	NSURLCredential				*credentialResult = nil;

	if ((IS_NOT_NULL(username)) && (IS_NOT_NULL(password)))
	{
		credentialResult = [NSURLCredential credentialWithUser: username
													  password: password
												   persistence: NSURLCredentialPersistenceForSession];
	}
	return (credentialResult);
}


//============================================================================
//	S4NetUtilities :: createQueryStringForPath:
//============================================================================
+ (NSString *)createQueryStringForPath: (NSString *)path, ...
{
	NSMutableArray				*queryStrArray;
	va_list						params;
	id							keyObject;
	id							valueObject;
	NSString					*keyStr;
	NSString					*valueStr;
	NSString					*queryArg;
	NSString					*queryStrings = nil;
	NSMutableString				*queryStrResult = nil;

	if (STR_NOT_EMPTY(path))
	{
		// init variable arguments
		va_start(params, path);

		// loop to pull key/value pairs out of the varargs and add to array of strings
		queryStrArray = [NSMutableArray array];
		while (YES)
		{
			keyObject = va_arg(params, id);
			if (IS_NULL(keyObject))
			{
				break;
			}

			valueObject = va_arg(params, id);
			if (IS_NULL(valueObject))
			{
				break;
			}

			keyStr = [keyObject description];
			valueStr = [valueObject description];
			queryArg = [NSString stringWithFormat: @"%@=%@", keyStr, valueStr];
			[queryStrArray addObject: queryArg];
		}

		// reset variable arguments
		va_end(params);

		// if there are two or more query arguments, join them with an "&"
		if ([queryStrArray count] > 0)
		{
			queryStrings = [queryStrArray componentsJoinedByString: @"&"];
			if (STR_NOT_EMPTY(queryStrings))
			{
				queryStrResult = [NSMutableString stringWithCapacity: 256];
				[queryStrResult appendFormat: @"%@%@", path, queryStrings];
			}
		}
	}
	return (queryStrResult);
}


//============================================================================
//	S4NetUtilities : getStringEncoding:forFileAtPath:
//============================================================================
+ (BOOL)getStringEncoding: (NSStringEncoding *)encodingPtr forHttpDocAtPath: (NSString *)path
{
	NSURL					*url;
	NSString				*tmpStr;
	NSError					*error = nil;
	BOOL					bResult = NO;
	
	if (nil != encodingPtr)
	{
		url = [S4NetUtilities createNSUrlForPathStr: path baseStr: nil];
		if (IS_NOT_NULL(url))
		{
			tmpStr = [NSString stringWithContentsOfURL: url usedEncoding: encodingPtr error: &error];
			if ((STR_NOT_EMPTY(tmpStr)) && (IS_NULL(error)))
			{
				bResult = YES;
			}
		}
	}
	return (bResult);
}

@end
