/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2012
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		S4TinyURLService.m
 * Module:		WebServices
 * Library:		S4 iOS Libraries
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import "S4TinyURLService.h"
#import "S4NetUtilities.h"
#import "S4CommonDefines.h"


// =================================== Defines =========================================



// ================================== Typedefs =========================================



// =================================== Globals =========================================



// ============================= Forward Declarations ==================================



// ================================== Inlines ==========================================



// ====================== Begin Class S4TinyURLService =======================

@implementation S4TinyURLService


//============================================================================
//	S4TinyURLService :: properties
//============================================================================



////////////////////////////////////////// CLASS METHODS //////////////////////////////////////////



//////////////////////////////////////// INSTANCE METHODS /////////////////////////////////////////


//============================================================================
//	S4TinyURLService :: init
//============================================================================
- (id)init
{
	self = [super init];
	if (nil != self)
	{
	}
	return (self);
}


//============================================================================
//	S4TinyURLService :: getTinyURLWithString:
//============================================================================
- (NSString *)getTinyURLWithString: (NSString *)requestURL
{
	NSString						*urlStr;
	NSURLRequest					*urlRequest;
	NSURLResponse					*response = nil;
	NSError							*error = nil;
	NSData							*data;
	NSString						*shortUrlStrResult = nil;

	if (STR_NOT_EMPTY(requestURL))
	{
		urlStr = [NSString stringWithFormat: @"http://tinyurl.com/api-create.php?url=%@", requestURL];
		if (STR_NOT_EMPTY(urlStr))
		{
			urlRequest = [S4NetUtilities createRequestForPath: urlStr
													 useCache: YES
											  timeoutInterval: 21.0
													 postData: nil
												   dataIsForm: NO
												handleCookies: YES];
			if (IS_NOT_NULL(urlRequest))
			{
				data = [NSURLConnection sendSynchronousRequest: urlRequest returningResponse: &response error: &error];
				if ((nil == error) && (IS_NOT_NULL(data)))
				{
					shortUrlStrResult = [[[NSString alloc] initWithData: data encoding: NSASCIIStringEncoding] autorelease];
				}
			}
		}
	}
	return (shortUrlStrResult);
}

@end
