//
//  S4JSONSerializerTest.m
//  YAJL
//
//  Created by Gabriel Handford on 7/19/09.
//  Copyright 2009 . All rights reserved.
//

#import "YAJLTestCase.h"

#import "S4YajlSerializer.h"

@interface S4JSONSerializerTest : YAJLTestCase {}
@end

@implementation S4JSONSerializerTest

- (void)testGen1
{
	S4YajlSerializer *gen = [[S4YajlSerializer alloc] initWithGenOptions:S4JSONSerializerOptionsBeautify indentString:@"  "];
	[gen startDictionary];
	[gen string:@"key2"];
	[gen startArray];
	[gen string:@"arrayValue1"];
	[gen bool:YES];
	[gen bool:NO];
	[gen null];
	[gen number:[NSNumber numberWithInteger:1]];
	[gen number:[NSNumber numberWithDouble:234234.234234]];
	[gen number:[NSDecimalNumber notANumber]];
	[gen endArray];
	[gen endDictionary];
	NSString *buffer = [gen buffer];
	[gen release];

	NSString *expected = [self loadString:@"gen_expected1"];

	GHTestLog(buffer);
	GHAssertEqualStrings(buffer, expected, nil);
}

- (void)testGen2
{
	S4YajlSerializer *gen = [[S4YajlSerializer alloc] init];
	[gen startDictionary];
	[gen string:@"key2"];
	[gen startArray];
	[gen string:@"arrayValue1"];
	[gen bool:YES];
	[gen bool:NO];
	[gen null];
	[gen number:[NSNumber numberWithInteger:1]];
	[gen number:[NSNumber numberWithDouble:234234.234234]];
	[gen number:[NSDecimalNumber notANumber]];
	[gen endArray];
	[gen endDictionary];
	NSString *buffer = [gen buffer];
	[gen clear];
	[gen release];

	NSString *expected = [self loadString:@"gen_expected2"];

	GHTestLog(buffer);
	GHAssertEqualStrings(buffer, expected, nil);
}

- (void)testGenObject1
{
	NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:
						  [NSArray arrayWithObjects:@"arrayValue1", [NSNumber numberWithBool:YES], [NSNumber numberWithBool:NO], [NSNull null],
						  [NSNumber numberWithInteger:1], [NSNumber numberWithDouble:234234.234234], [NSDecimalNumber notANumber], nil], @"key2",
						  nil];
  
  S4YajlSerializer *gen = [[S4YajlSerializer alloc] initWithGenOptions:S4JSONSerializerOptionsBeautify indentString:@"  "];
  [gen object:dict];
  NSString *buffer = [gen buffer];
  [gen release];
  
  NSString *expected = [self loadString:@"gen_expected1"];  
  GHTestLog(buffer);
  GHAssertEqualStrings(buffer, expected, nil);  
}

- (void)testGenObjectUnknownType {
  NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:[NSDate dateWithTimeIntervalSince1970:1], @"date", nil];  
  S4YajlSerializer *gen = [[S4YajlSerializer alloc] init];
  GHAssertThrows([gen object:dict], nil);
  [gen release];
}

- (void)testGenObjectIgnoreUnknownType {
  NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:[NSDate dateWithTimeIntervalSince1970:1], @"date", nil];
  
  S4YajlSerializer *gen = [[S4YajlSerializer alloc] initWithGenOptions:S4JSONSerializerOptionsIgnoreUnknownTypes indentString:@""];
  [gen object:dict];
  NSString *buffer = [gen buffer];
  [gen release];
  
  NSString *expected = [self loadString:@"gen_expected_ignore_unknown1"]; 
  GHTestLog(buffer);
  GHAssertEqualStrings(buffer, expected, nil);  
}

- (void)testGenObjectPListTypes {
  const char *testData = "ABCDEFG";
  NSArray *array = [NSArray arrayWithObjects:
                    [NSDate dateWithTimeIntervalSince1970:1], 
                    [NSData dataWithBytes:testData length:6],
                    [NSURL URLWithString:@"http://www.yelp.com/"],
                    nil];
  
  S4YajlSerializer *gen = [[S4YajlSerializer alloc] initWithGenOptions:S4JSONSerializerOptionsIncludeUnsupportedTypes indentString:@""];
  [gen object:array];
  NSString *buffer = [gen buffer];
  [gen release];
  
  NSString *expected = [self loadString:@"gen_expected_plist1"];  
  GHTestLog(buffer);
  GHAssertEqualStrings(buffer, expected, nil);  
}

@end
