//
//  S4JSONParserTest.m
//  YAJL
//
//  Created by Gabriel Handford on 6/14/09.
//  Copyright 2009. All rights reserved.
//


@interface S4JSONParserTest : YAJLTestCase {}
@end

@implementation S4JSONParserTest

- (void)test {  
  S4YajlParser *parser = [[S4YajlParser alloc] init];
  [parser parse:[self loadData:@"example"]];
  
  NSError *error = [parser parserError];
  if (error) {
    NSLog(@"Parse error:\n%@", error);
    GHFail(@"Error: %@", error);
  }
  
  [parser release];
}

- (void)testError {
  S4YajlParser *parser = [[S4YajlParser alloc] initWithParserOptions:0];
  S4JSONParserError status = [parser parse:[self loadData:@"error"]];
  GHAssertTrue(status == S4JSONParserParsingError, @"Should have error status");
  
  NSError *error = [parser parserError];
  if (error) {
    GHTestLog(@"Parse error:\n%@", error);    
  } else {
    GHFail(@"Should have error");
  }
  
  [parser release];
}

- (void)testError2 {
  S4YajlParser *parser = [[S4YajlParser alloc] initWithParserOptions:0];
  S4JSONParserError status = [parser parse:[self loadData:@"error2"]];
  GHAssertTrue(status == S4JSONParserParsingError, @"Should have error status");
  
  NSError *error = [parser parserError];
  if (error) {
    GHTestLog(@"Parse error:\n%@", error);    
    GHAssertTrue([[error localizedDescription] hasPrefix:@"parse error: invalid object key (must be a string)\n"], nil);
  } else {
    GHFail(@"Should have error");
  }
  
  [parser release];
}

- (void)testComments {
  S4YajlParser *parser = [[S4YajlParser alloc] initWithParserOptions:S4JSONParserOptionsAllowComments];
  S4JSONParserError status = [parser parse:[self loadData:@"comments"]];
  GHAssertTrue(status == S4JSONParserNoError, @"Should have OK status");
  
  NSError *error = [parser parserError];
  if (error) {
    GHTestLog(@"Parse error:\n%@", error);    
    GHFail(@"Error: %@", error);
  }
  
  [parser release];
}

- (void)testFailOnComments {
  S4YajlParser *parser = [[S4YajlParser alloc] initWithParserOptions:0];
  S4JSONParserError status = [parser parse:[self loadData:@"comments"]];
  GHAssertTrue(status == S4JSONParserParsingError, @"Should have error status");
  
  NSError *error = [parser parserError];
  if (error) {
    GHTestLog(@"Parse error:\n%@", error);    
  } else {
    GHFail(@"Should have error");
  }

  [parser release];
}

- (void)testDoubleOverflow {
  S4YajlParser *parser = [[S4YajlParser alloc] init];
  S4JSONParserError status = [parser parse: [self loadData: @"overflow"]];
  GHAssertEquals(status, S4JSONParserCanceledError, @"Should have error status");
  
  NSError *error = [parser parserError];
  if (error) {
    GHTestLog(@"Parse error:\n%@", error);
    GHAssertEquals([error code], (NSInteger)S4JSONParserDoubleOverflowError, nil);
    GHAssertEqualStrings([[error userInfo] objectForKey: S4YajlParserValueKey], @"1.79769e+309", nil);    
  } else {
    GHFail(@"Should have error");
  }
  
  [parser release];
}

- (void)testLongLongOverflow
{
  S4YajlParser *parser = [[S4YajlParser alloc] initWithParserOptions:S4JSONParserOptionsStrictPrecision];
  S4JSONParserError status = [parser parse:[self loadData:@"overflow_longlong"]];
  GHAssertEquals(status, S4JSONParserCanceledError, @"Should have error status");
  
  NSError *error = [parser parserError];
  if (error) {
    GHTestLog(@"Parse error:\n%@", error);
    GHAssertEquals([error code], (NSInteger)S4JSONParserIntegerOverflowError, nil);
//  GHAssertEqualStrings([[error userInfo] objectForKey: S4YajlParserValueKey], @"9223372036854775808", nil);
    GHAssertEqualStrings([[error userInfo] objectForKey: S4YajlParserValueKey], @"9223372036854775807", nil);
  } else {
    GHFail(@"Should have error");
  }
  
  [parser release];
}

- (void)testOverflow2 {
  S4YajlParser *parser = [[S4YajlParser alloc] initWithParserOptions:0];
  S4JSONParserError status = [parser parse:[self loadData:@"overflow2"]];
  GHAssertTrue(status == S4JSONParserNoError, @"Should have ok status");
  [parser release];
}

- (void)testStreaming
{
	S4YajlParser *parser = [[S4YajlParser alloc] initWithParserOptions:0];
	S4JSONParserError status;
	status = [parser parse:[self loadData:@"stream1"]];
	GHAssertTrue(status == S4JSONParserNoError, @"Should have insufficient data");

	status = [parser parse:[self loadData:@"stream2"]];
	GHAssertTrue(status == S4JSONParserNoError, @"Should have finished");

	status = [parser parseCompleted];
	GHAssertTrue(status == S4JSONParserNoError, @"Have finished");
	[parser release];
}

// TODO(gabe): Should error if you try to re-use
- (void)_testParseMoreThanOnce {
  S4YajlParser *parser = [[S4YajlParser alloc] initWithParserOptions:0];
  parser.delegate = self;
  S4JSONParserError status;
  status = [parser parse:[self loadData:@"example"]];
  GHAssertTrue(status == S4JSONParserNoError, @"Should be OK");
  status = [parser parse:[self loadData:@"example"]];
  GHAssertTrue(status == S4JSONParserParsingError, @"Should error");
	[parser release];
}

@end