//
//  CPropertyFactory.h
//  Test
//
//  Created by Jonathan Wight on 10/3/11.
//  Copyright 2011 Jonathan Wight. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are
//  permitted provided that the following conditions are met:
//
//     1. Redistributions of source code must retain the above copyright notice, this list of
//        conditions and the following disclaimer.
//
//     2. Redistributions in binary form must reproduce the above copyright notice, this list
//        of conditions and the following disclaimer in the documentation and/or other materials
//        provided with the distribution.
//
//  THIS SOFTWARE IS PROVIDED BY 2011 TOXICSOFTWARE.COM ``AS IS'' AND ANY EXPRESS OR IMPLIED
//  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
//  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 2011 TOXICSOFTWARE.COM OR
//  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
//  ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//  The views and conclusions contained in the software and documentation are those of the
//  authors and should not be interpreted as representing official policies, either expressed
//  or implied, of 2011 toxicsoftware.com.


// ================================== Includes =========================================

#import <Foundation/Foundation.h>
#import <objc/runtime.h>


// =================================== Defines =========================================



// ================================== Typedefs =========================================

typedef id (^PropertyGetterBlock)(id _self, NSString *key);

typedef void (^PropertySetterBlock)(id _self, NSString *key, id value);

typedef NS_ENUM(NSUInteger, CPropertyFactoryError)
{
	CPropertyFactoryNoError					= 0,
	CPropertyFactoryNoGetterError			= 1,
	CPropertyFactoryNoGetterImplError		= 2,
	CPropertyFactoryNoSetterError			= 3,
	CPropertyFactoryNoSetterImplError		= 4,
	CPropertyFactoryUnknownError			= 5,
	CPropertyFactoryUnimplementedError		= 6
};


// =================================== Globals =========================================

extern NSString * const						CPropertyFactoryErrorDomain;


// ============================= Forward Declarations ==================================



// ================================== Protocols ========================================


// ============================= Class CPropertyFactory ================================

@interface CPropertyFactory : NSObject

- (BOOL)configure: (Class)inClass
		   getter: (PropertyGetterBlock)inGetter
		   setter: (PropertySetterBlock)inSetter
			error: (NSError **)outError;

- (BOOL)configure: (Class)inClass
	 propertyName: (NSString *)inProperty
		   getter: (PropertyGetterBlock)inGetter
		   setter: (PropertySetterBlock)inSetter
			error: (NSError **)outError;

/** Add property getter & setter implementations to class 'inClass' for the property 'inProperty'. */
- (BOOL)configure: (Class)inClass
		 property: (objc_property_t)inProperty
		   getter: (PropertyGetterBlock)inGetter
		   setter: (PropertySetterBlock)inSetter
			error: (NSError **)outError;

@end
