//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
//
//
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iPhone Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2009
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		SHKApplicationSettings.m
 * Module:		Configuration
 * Library:		ShareKit
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import "SHKApplicationSettings.h"
#import "SHKDefaultAppSettingsDelegate.h"
#import "S4SingletonClass.h"
#import "S4CommonDefines.h"


// =================================== Defines =========================================



// ================================== Typedefs =========================================



// =================================== Globals =========================================



// ============================= Forward Declarations ==================================



// ================================== Inlines ==========================================



// ===================== Begin Class SHKApplicationSettings () =========================



// ================ Begin Class SHKApplicationSettings (PrivateImpl) ===================

@interface SHKApplicationSettings (PrivateImpl)

- (void)oneTimeInit;

@end



@implementation SHKApplicationSettings (PrivateImpl)

- (void)oneTimeInit
{
	m_delegate = [[SHKDefaultAppSettingsDelegate alloc] init];
}

@end




// ======================== Begin Class SHKApplicationSettings =========================


@implementation SHKApplicationSettings

///////////////////////////////////// START SINGLETON METHODS /////////////////////////////////////


SYNTHESIZE_SINGLETON_CLASS(SHKApplicationSettings)


//////////////////////////////////////// INSTANCE METHODS /////////////////////////////////////////

- (BOOL)setDelegate: (id <SHKApplicationSettingsDelegate>)newDelegate
{
	BOOL				bResult = NO;

	if (IS_NOT_NULL(newDelegate))
	{
		S4_SAFE_RETAIN(m_delegate, newDelegate)
		bResult = YES;
	}
	return (bResult);
}


- (id)configurationValue: (NSString *)selectorStr
{
	SEL						selector;
	NSString				*retValue = nil;

	selector = NSSelectorFromString(selectorStr);
	if ([m_delegate respondsToSelector: selector])
	{
		retValue = [m_delegate performSelector: selector];
	}
	return (retValue);
}

@end
