/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iOS Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2010
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		SHKConfig.h
 * Module:		Configuration
 * Library:		ShareKit
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import "S4CommonDefines.h"



// =================================== Defines =========================================
#define SHKLog	S4DebugLog
#define SHKDebugShowLogs 1


// ================================== Typedefs =========================================



// =================================== Globals =========================================

/*
 API Keys
 --------
 This is the longest step to getting set up, it involves filling in API keys for the supported services.
 It should be pretty painless though and should hopefully take no more than a few minutes.
 
 Each key below as a link to a page where you can generate an api key.  Fill in the key for each service below.
 
 A note on services you don't need:
 If, for example, your app only shares URLs then you probably won't need image services like Flickr.
 In these cases it is safe to leave an API key blank.
 
 However, it is STRONGLY recommended that you do your best to support all services for the types of sharing you support.
 The core principle behind ShareKit is to leave the service choices up to the user.  Thus, you should not remove any services,
 leaving that decision up to the user.
 */

/*	Facebook settings to get right:
 
	URL Schemes
	---
	You must create a URL scheme in your Info.plist that is in the format fb[app_id]. See the documentation
	on the iOS SDK under Authentication and Authorization for more details. This is to allow
	the new Single Sign-on capabilities of the iOS SDK to callback to your application, should
	it use fast app switching to authenticate in the Facebook app or Safari.
 
	Modify AppDelegate class
	---
	You must implement the application:handleOpenURL: method in your AppDelegate class. In this method, call
	the handleOpenURL: method on the facebook property of an SHKFacebook instance.
	For example:

	- (BOOL)application: (UIApplication *)application handleOpenURL: (NSURL *)url
	{
		SHKFacebook *facebookSharer = [[[SHKFacebook alloc] init] autorelease];
		return [[facebookSharer facebook] handleOpenURL:url];
	}
*/

/*	Twitter settings to get right:
 
	Differences between OAuth and xAuth
	--
	There are two types of authentication provided for Twitter, OAuth and xAuth.  OAuth is the default and will
	present a web view to log the user in.  xAuth presents a native entry form but requires Twitter to add xAuth
	to your app (you have to request it from them).  If your app has been approved for xAuth, set
	SHKTwitterUseXAuth to 1.

	Callback URL (important to get right for OAuth users)
	--
	1. Open your application settings at dev.twitter.com/apps/
	2. 'Application Type' should be set to BROWSER (not client)
	3. 'Callback URL' should match whatever you enter in SHKTwitterCallbackUrl.  The callback url doesn't have to
	be an actual existing url.  The user will never get to it because ShareKit intercepts it before the user is
	redirected.  It just needs to match.
*/


// ============================= Forward Declarations ==================================


