/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the S4 iPhone Libraries.
 *
 * The Initial Developer of the Original Code is
 * Michael Papp dba SeaStones Software Company.
 * All software created by the Initial Developer are Copyright (C) 2008-2009
 * the Initial Developer. All Rights Reserved.
 *
 * Original Author:
 *			Michael Papp, San Francisco, CA, USA
 *
 * ***** END LICENSE BLOCK ***** */

/* ***** FILE BLOCK ******
 *
 * Name:		SHKDefaultAppSettingsDelegate.h
 * Module:		Configuration
 * Library:		ShareKit
 *
 * ***** FILE BLOCK *****/


// ================================== Includes =========================================

#import <Foundation/Foundation.h>
#import "SHKApplicationSettings.h"


// =================================== Defines =========================================



// ================================== Typedefs =========================================



// =================================== Globals =========================================



// ============================= Forward Declarations ==================================



// ================================== Protocols ========================================



// ===================== Class SHKDefaultAppSettingsDelegate ===========================

@interface SHKDefaultAppSettingsDelegate : NSObject <SHKApplicationSettingsDelegate>
{

}

// App Description - these values are used by any service that shows 'shared from XYZ'
- (NSString *)appName;
- (NSString *)appURL;

// Posterous  -  apidocs.posterous.com/
- (NSString *)posterousAPIKey;

// Delicious  -  developer.apps.yahoo.com/projects
- (NSString *)deliciousConsumerKey;
- (NSString *)deliciousSecretKey;

// Read It Later  -  readitlaterlist.com/api/?shk
- (NSString *)readItLaterKey;

// Evernote  -  www.evernote.com/about/developer/api/
- (NSString *)evernoteUserStoreURL;
- (NSString *)evernoteSecretKey;
- (NSString *)evernoteConsumerKey;
- (NSString *)evernoteNetStoreURLBase;

// Bit.ly  -  bit.ly/account/register, after signup: bit.ly/a/your_api_key
- (NSString *)bitLyLogin;
- (NSString *)bitLyKey;

// Flickr  -  www.flickr.com/services/apps/create/
- (NSString *)flickrConsumerKey;
- (NSString *)flickrSecretKey;
- (NSString *)flickrCallbackUrl;

// Facebook  -  www.facebook.com/developers
- (NSString *)facebookAppId;
- (NSString *)facebookSecret;
- (NSNumber *)openURLsInExternalBrowser;

// Twitter  -  dev.twitter.com/apps/new
- (NSString *)twitterConsumerKey;
- (NSString *)twitterSecret;
// You need to set this if using OAuth, see note above (xAuth users can skip it)
- (NSString *)twitterCallbackUrl;
// To use xAuth, set to 1
- (NSNumber *)twitterUseXAuth;
// Enter your app's twitter account if you'd like to ask the user to follow it when logging in. (Only for xAuth)
- (NSString *)twitterUsername;


- (NSString *)barStyle;
// Value between 0-255, set all to -1 for default
- (NSNumber *)barTintColorRed;
// Value between 0-255, set all to -1 for default
- (NSNumber *)barTintColorGreen;
// Value between 0-255, set all to -1 for default
- (NSNumber *)barTintColorBlue;

//	Forms
// Value between 0-255, set all to -1 for default
- (NSNumber *)formFontColorRed;
// Value between 0-255, set all to -1 for default
- (NSNumber *)formFontColorGreen;
// Value between 0-255, set all to -1 for default
- (NSNumber *)formFontColorBlue;
// Value between 0-255, set all to -1 for default
- (NSNumber *)formBgColorRed;
// Value between 0-255, set all to -1 for default
- (NSNumber *)formBgColorGreen;
// Value between 0-255, set all to -1 for default
- (NSNumber *)formBgColorBlue;

//	iPad Views
- (NSString *)modalPresentationStyle;
- (NSString *)modalTransitionStyle;

// ShareMenu Ordering - setting this to 1 will show list in Alphabetical Order, setting to 0 will follow the order in SHKShares.plist
- (NSNumber *)shareMenuAlphabeticalOrder;

// Append 'Shared With 'Signature to Email (and related forms)
- (NSNumber *)sharedWithSignature;

// Name of .plist file holding the list of sharer classes
- (NSString *)sharersPlistName;

//	Advanced UI Configuration - these settings only need to be changed for uber custom installs
- (NSNumber *)maxFavCount;
- (NSString *)favsPrefixKey;
- (NSString *)authPrefix;

// Global setting for allowing offline activities for services - default is YES
- (NSNumber *)allowOffline;

@end
