//
//  SHKFacebook.m
//  ShareKit
//
//  Created by Colin Humber on 1/25/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "SHKFacebook.h"
#import "SHKApplicationSettings.h"
#import "S4CommonDefines.h"


static S4_INTERN_CONSTANT_NSSTR				SHKFacebookAccessToken = @"SHKFacebookAccessToken";
static S4_INTERN_CONSTANT_NSSTR				SHKFacebookExpirationDate = @"SHKFacebookExpirationDate";
static S4_INTERN_CONSTANT_NSSTR				SHKFacebookPendingItem = @"SHKFacebookPendingItem";
static S4_INTERN_CONSTANT_NSSTR				SHKFacebookUserInfo = @"SHKFacebookUserInfo";
static Facebook								*g_facebook = nil;


@interface SHKFacebook()

- (void)showFacebookForm;

@end



@interface SHKFacebook (privateImpl)

+ (Facebook *)getGlobalFacebookforSharer: (SHKFacebook *)newSharer;
+ (Facebook *)getGlobalFacebookWithoutSharer;
+ (void)flushAccessToken;
+ (NSString *)imagePathForStoredImage: (UIImage*)image;
+ (UIImage *)getImageStoredAtPath: (NSString *)imagePath;
+ (NSDate *)getExpirationDate;
+ (void)setExpirationDate;
- (Facebook *)facebookForInstance;

@end



@implementation SHKFacebook (privateImpl)

+ (Facebook *)getGlobalFacebookforSharer: (SHKFacebook *)newSharer 
{
	SHKFacebook					*prevSharer;

	@synchronized([SHKFacebook class])
	{
		if (nil == g_facebook)
		{
			g_facebook = [[Facebook alloc] initWithAppId: SHKCONFIG(facebookAppId) andDelegate: newSharer];
		}
		else
		{
			// if there is a current sessionDelegate and it is not us, maintain retain count
			prevSharer = (SHKFacebook *)g_facebook.sessionDelegate;
			if (prevSharer != newSharer)
			{
				if (IS_NOT_NULL(prevSharer))
				{
					[prevSharer retain];
				}
				g_facebook.sessionDelegate = newSharer;
			}
		}
	}
	return (g_facebook);
}


+ (Facebook *)getGlobalFacebookWithoutSharer
{
	SHKFacebook					*sharerResult;
	
	if ((nil != g_facebook) && (IS_NOT_NULL(g_facebook.sessionDelegate)))
	{
		return (g_facebook);
	}
	else
	{
		sharerResult = [[[SHKFacebook alloc] init] autorelease];
		return ([SHKFacebook getGlobalFacebookforSharer: sharerResult]);
	}
}


+ (void)flushAccessToken 
{
	if (nil != g_facebook)
	{
		g_facebook.accessToken = nil;
		g_facebook.expirationDate = nil;
	}

	[SHK removeAuthValueForKey: SHKFacebookAccessToken forSharer: [self sharerId]];
	[[NSUserDefaults standardUserDefaults] removeObjectForKey: SHKFacebookExpirationDate];
	[[NSUserDefaults standardUserDefaults] removeObjectForKey: SHKFacebookUserInfo];
	[[NSUserDefaults standardUserDefaults] synchronize];
}


+ (NSString *)imagePathForStoredImage: (UIImage *)image
{
	NSFileManager					*fileManager;
	NSArray							*paths;
	NSString						*cachePath;
	NSString						*filePath;
	NSString						*fileName;
	NSData							*imageData;
	NSString						*imagePath = nil;
	
	fileManager = [[NSFileManager alloc] init];
	paths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
	cachePath = [paths objectAtIndex: 0];
	filePath = [cachePath stringByAppendingPathComponent: @"SHKImage"];

	// Check if the path exists, otherwise create it
	if (![fileManager fileExistsAtPath: filePath])
	{
		[fileManager createDirectoryAtPath: filePath withIntermediateDirectories: YES attributes: nil error: nil];
	}

	fileName = [NSString stringWithFormat: @"img-%i-%i", [[NSDate date] timeIntervalSince1970], arc4random()];
	imagePath = [filePath stringByAppendingPathComponent: fileName];

	// store image in cache
	imageData = UIImagePNGRepresentation(image);
	[imageData writeToFile: imagePath atomically: YES];
	[fileManager release];
	return (imagePath);
}


+ (UIImage*)getImageStoredAtPath: (NSString*)imagePath
{
	NSData *imageData = [NSData dataWithContentsOfFile:imagePath];
	UIImage *image = nil;
	if (imageData) {
		image = [UIImage imageWithData:imageData];
	}
	// Unlink the stored file:
	[[[NSFileManager alloc] init] removeItemAtPath: imagePath error: nil];
	return image;
}


+ (NSDate *)getExpirationDate
{
	return ((NSDate *)[[NSUserDefaults standardUserDefaults] objectForKey: SHKFacebookExpirationDate]);
}


+ (void)setExpirationDate
{
	[[NSUserDefaults standardUserDefaults] setObject: [[SHKFacebook getGlobalFacebookWithoutSharer] expirationDate]
											  forKey: SHKFacebookExpirationDate];
	[[NSUserDefaults standardUserDefaults] synchronize];
}


- (Facebook *)facebookForInstance
{
	return ([SHKFacebook getGlobalFacebookforSharer: self]);
}

@end




@implementation SHKFacebook

@synthesize pendingFacebookAction;


+ (BOOL)handleOpenURL: (NSURL*)url 
{
	Facebook				*fb;

	fb = [SHKFacebook getGlobalFacebookWithoutSharer];
	return ([fb handleOpenURL: url]);
}


- (id)init
{
	if ((self = [super init]))
	{
		permissions = [[NSArray alloc] initWithObjects: @"publish_stream", @"offline_access", nil];
	}
	return (self);
}


- (void)dealloc
{
	if ((nil != g_facebook) && (g_facebook.sessionDelegate == self))
	{
		g_facebook.sessionDelegate = nil;
	}

	[super dealloc];
}




#pragma mark -
#pragma mark Configuration : Service Definition

+ (NSString *)sharerTitle
{
	return @"Facebook";
}


+ (BOOL)canShareURL
{
	return YES;
}


+ (BOOL)canShareText
{
	return YES;
}


+ (BOOL)canShareImage
{
	return YES;
}


+ (BOOL)canShareOffline
{
	return NO;  // TODO - would love to make this work
}


+ (BOOL)canGetUserInfo
{
    return YES;
}



#pragma mark -
#pragma mark Configuration : Dynamic Enable

- (BOOL)shouldAutoShare
{
	return NO;
}

#pragma mark -
#pragma mark Authentication

- (BOOL)isAuthorized
{
	Facebook					*fb;
	BOOL						bResult = NO;

	fb = [self facebookForInstance];
	bResult = [fb isSessionValid];
	if (NO == bResult)
	{
		fb.accessToken = [self getAuthValueForKey: SHKFacebookAccessToken];
		fb.expirationDate = [SHKFacebook getExpirationDate];
		bResult = [fb isSessionValid];
	}
	return (bResult);
}


- (void)promptAuthorization
{
	NSMutableDictionary						*itemDict;

	// store the pending item in NSUserDefaults as the authorize could kick the user out to the Facebook app or Safari
	itemDict = [NSMutableDictionary dictionaryWithDictionary: [self.item dictionaryRepresentation]];
	if (IS_NOT_NULL(item.image))
	{
		[itemDict setObject: [SHKFacebook imagePathForStoredImage: item.image] forKey: @"imagePath"];
	}
	[[NSUserDefaults standardUserDefaults] setObject: itemDict forKey: SHKFacebookPendingItem];
	[[NSUserDefaults standardUserDefaults] synchronize];

#ifdef SHKFacebookLocalAppID
	[[self facebookForInstance] authorize: permissions localAppId: SHKFacebookLocalAppID];
#else
	[[self facebookForInstance] authorize: permissions];
#endif

}


- (void)authFinished: (SHKRequest *)aRequest
{
	[self authDidFinish: aRequest.success];
}


+ (void)logout
{
	Facebook					*fb;
	SHKFacebook					*sharer;

	fb = [SHKFacebook getGlobalFacebookWithoutSharer];
	sharer = (SHKFacebook *)fb.sessionDelegate;
	if (IS_NOT_NULL(sharer))
	{
		fb.accessToken = [sharer getAuthValueForKey: SHKFacebookAccessToken];
		fb.expirationDate = [SHKFacebook getExpirationDate];
	}
	[fb logout: sharer];

	// clean up
	[[NSUserDefaults standardUserDefaults] removeObjectForKey: SHKFacebookPendingItem];
	[self flushAccessToken];
	fb.accessToken = nil;
	fb.expirationDate = nil;
}



#pragma mark -
#pragma mark Share API Methods

- (BOOL)send
{
	NSMutableDictionary					*params;
	NSString							*actionLinks;
	Facebook							*instanceFacebook;
	NSString							*graphPath;
	BOOL								bSendingPhoto = NO;

 	if (![self validateItem])
		return NO;

	params = [NSMutableDictionary dictionary];
	actionLinks = [NSString stringWithFormat: @"{\"name\":\"Get %@\",\"link\":\"%@\"}", SHKEncode(SHKCONFIG(appName)), SHKEncode(SHKCONFIG(appURL))];
	[params setObject: actionLinks forKey: @"actions"];

	instanceFacebook = [self facebookForInstance];

	if ((item.shareType == SHKShareTypeURL) && (IS_NOT_NULL(item.URL)))
	{
		NSString *url = [item.URL absoluteString];
		[params setObject: url forKey: @"link"];

		if (IS_NOT_NULL(item.title))
		{
			[params setObject: item.title forKey: @"name"];
		}
		else
		{
			[params setObject: url forKey: @"name"];
		}

		if (IS_NOT_NULL(item.text))
		{
			[params setObject: item.text forKey: @"message"];
		}

		NSString *pictureURI = [item customValueForKey: @"picture"];
		if (IS_NOT_NULL(pictureURI))
		{
			[params setObject: pictureURI forKey: @"picture"];
		}

		graphPath = @"me/feed";
	}
	else if ((item.shareType == SHKShareTypeText) && (IS_NOT_NULL(item.text)))
	{
		[params setObject: item.text forKey: @"message"];
		graphPath = @"me/feed";
	}
	else if ((item.shareType == SHKShareTypeImage) && (IS_NOT_NULL(item.image)))
	{
		[params setObject: item.image forKey: @"picture"];

		if (IS_NOT_NULL(item.title))
		{
			[params setObject: item.title forKey: @"caption"];
        }

		if (IS_NOT_NULL(item.text))
		{
			[params setObject: item.text forKey: @"message"];
		}
		graphPath = @"me/photos";
		bSendingPhoto = YES;
	}
    else if (item.shareType == SHKShareTypeUserInfo)
    {
        [self setQuiet: YES];
        [instanceFacebook requestWithGraphPath: @"me" andDelegate: self];
        return YES;
    }
	else
	{
		return (NO);
	}

	if ((YES == [self shouldAutoShare]) || (YES == bSendingPhoto))
	{
		[instanceFacebook requestWithGraphPath: graphPath 
									 andParams: params 
								 andHttpMethod: @"POST" 
								   andDelegate: self];
            [self sendDidStart];
        }
	else
	{
		[instanceFacebook dialog: @"feed" andParams: params andDelegate: self];
	}
	return (YES);
}
	

- (void)show
{
    if (item.shareType == SHKShareTypeText)        
    {
        [self showFacebookForm];
    }
 	else
    {
        [self tryToSend];
    }
}


- (void)showFacebookForm
{
 	SHKFormControllerLargeTextField *rootView = [[SHKFormControllerLargeTextField alloc] initWithNibName:nil bundle:nil delegate:self];  
 	// force view to load so we can set textView text
 	[rootView view];
 	rootView.textView.text = item.text;
    self.navigationBar.tintColor = [UIColor colorWithRed:59.0f/255 green:89.0f/255 blue:152.0f/255 alpha:1];;
 	[self pushViewController:rootView animated:NO];
    [rootView release];
    [[SHK currentHelper] showViewController:self];  
}


- (void)sendForm:(SHKFormControllerLargeTextField *)form
{  
 	self.item.text = form.textView.text;
 	[self tryToSend];
}  



#pragma mark FBDialogDelegate methods

- (void)dialogDidComplete: (FBDialog *)dialog
{
	if (pendingFacebookAction == SHKFacebookPendingStatus)
	{
		[self sendDidFinish];
	}
}


- (void)dialogCompleteWithUrl: (NSURL *)url 
{
	NSRange					errorRange;

  // error_code=190: user changed password or revoked access to the application,
  // so spin the user back over to authentication :
	errorRange = [[url absoluteString] rangeOfString: @"error_code=190"];
	if (errorRange.location != NSNotFound) 
	{
		[SHKFacebook flushAccessToken];
		[self authorize];
	}
}


- (void)dialogDidCancel: (FBDialog *)dialog
	{
		[self sendDidCancel];
	}


- (void)dialog: (FBDialog *)dialog didFailWithError: (NSError *)error 
{
	if (error.code != NSURLErrorCancelled)
	{
		[self sendDidFailWithError: error];
	}
}


- (void)dialogDidNotComplete: (FBDialog *)dialog
{
	if (pendingFacebookAction == SHKFacebookPendingStatus)
	{
		[self sendDidCancel];
	}
}


- (BOOL)dialog: (FBDialog *)dialog shouldOpenURLInExternalBrowser: (NSURL *)url
{
	return ([SHKCONFIG(openURLsInExternalBrowser) boolValue]);
}



#pragma mark FBSessionDelegate methods

- (void)fbDidLogin
{
	NSDictionary					*storedItem;
	NSString						*imagePath;

	// store the Facebook credentials for use in future requests
	[SHK setAuthValue: [[SHKFacebook getGlobalFacebookforSharer: self] accessToken] forKey: SHKFacebookAccessToken forSharer: [self sharerId]];
	[SHKFacebook setExpirationDate];
	
	// we might have been kicked out to Safari so retrieve the shared item from user defaults
	storedItem = [[NSUserDefaults standardUserDefaults] objectForKey: SHKFacebookPendingItem];
	if (IS_NOT_NULL(storedItem))
	{
		self.item = [SHKItem itemFromDictionary: storedItem];
		imagePath = [storedItem objectForKey: @"imagePath"];
		if (nil != imagePath)
		{
			self.item.image = [SHKFacebook getImageStoredAtPath: imagePath];
		}
		[[NSUserDefaults standardUserDefaults] removeObjectForKey: SHKFacebookPendingItem];
		[[NSUserDefaults standardUserDefaults] synchronize];
	}
	
    // -- Try to share again
    if ([self validateItem])  //only share if we have an item -  wont have item if only logging in
	{
		[self share];
		[self authDidFinish:true];
	}
}


- (void)fbDidNotLogin: (BOOL)cancelled
{
	// not handling this
}


- (void)fbDidLogout
{
	// not handling this
}



#pragma mark FBRequestDelegate methods

- (void)requestLoading: (FBRequest *)request
{
  [self sendDidStart];
}


- (void)request: (FBRequest *)aRequest didLoad: (id)result
{
	if ([result objectForKey:@"username"])
	{        
        [[NSUserDefaults standardUserDefaults] setObject: result forKey: SHKFacebookUserInfo];
    }     

	[self sendDidFinish];
}


- (void)request: (FBRequest *)aRequest didFailWithError: (NSError *)error
{
	[self sendDidFailWithError: error];
}

@end
